import sys

# sys.platform.startswith(("freebsd", "midnightbsd", "openbsd", "netbsd")):
if sys.platform != "linux" and sys.platform != "win32" and sys.platform != "darwin":
    from collections.abc import Sequence
    from socket import AddressFamily, SocketKind
    from typing import Final, overload

    AF_LINK: Final[int]
    RLIMIT_AS: Final[int]  # only FreeBSD
    RLIMIT_CORE: Final[int]  # only FreeBSD
    RLIMIT_CPU: Final[int]  # only FreeBSD
    RLIMIT_DATA: Final[int]  # only FreeBSD
    RLIMIT_FSIZE: Final[int]  # only FreeBSD
    RLIMIT_MEMLOCK: Final[int]  # only FreeBSD
    RLIMIT_NOFILE: Final[int]  # only FreeBSD
    RLIMIT_NPROC: Final[int]  # only FreeBSD
    RLIMIT_RSS: Final[int]  # only FreeBSD
    RLIMIT_STACK: Final[int]  # only FreeBSD
    RLIMIT_SWAP: Final[int]  # only FreeBSD
    RLIMIT_SBSIZE: Final[int]  # only FreeBSD
    RLIMIT_NPTS: Final[int]  # only FreeBSD
    RLIM_INFINITY: Final[int]  # only FreeBSD

    def getpagesize() -> int: ...
    def net_if_addrs() -> list[tuple[str, int, str, str | None, str | None, str | None]]: ...
    def net_if_flags(nic_name: str, /) -> list[str]: ...
    def net_if_is_running(nic_name: str, /) -> bool: ...
    def net_if_mtu(nic_name: str, /) -> int: ...
    def proc_priority_get(pid: int, /) -> int: ...
    def proc_priority_set(pid: int, priority: int, /) -> None: ...
    def net_if_duplex_speed(nic_name: str, /) -> tuple[int, int]: ...  # It's actually list of 2 elements
    def proc_is_zombie(pid: int, /) -> bool: ...

    version: Final[int]
    SIDL: Final[int]
    SRUN: Final[int]
    SSLEEP: Final[int]
    SSTOP: Final[int]
    SZOMB: Final[int]
    SWAIT: Final[int]  # only FreeBSD
    SLOCK: Final[int]  # only FreeBSD
    SDEAD: Final[int]  # only OpenBSD and NetBSD
    SONPROC: Final[int]  # only OpenBSD and NetBSD
    SSUSPENDED: Final[int]  # only NetBSD
    TCPS_CLOSED: Final[int]
    TCPS_CLOSING: Final[int]
    TCPS_CLOSE_WAIT: Final[int]
    TCPS_LISTEN: Final[int]
    TCPS_ESTABLISHED: Final[int]
    TCPS_SYN_SENT: Final[int]
    TCPS_SYN_RECEIVED: Final[int]
    TCPS_FIN_WAIT_1: Final[int]
    TCPS_FIN_WAIT_2: Final[int]
    TCPS_LAST_ACK: Final[int]
    TCPS_TIME_WAIT: Final[int]
    PSUTIL_CONN_NONE: Final = 128

    def proc_cmdline(pid: int, /) -> list[str]: ...
    def proc_cwd(pid: int, /) -> str: ...
    def proc_environ(pid: int, /) -> dict[str, str]: ...
    def proc_name(pid: int, /) -> str: ...
    def proc_num_fds(pid: int, /) -> int: ...
    def proc_oneshot_info(
        pid: int, /
    ) -> tuple[
        int,
        int,
        int,
        int,
        int,
        int,
        int,
        int,
        int,
        float,
        int,
        int,
        int,
        int,
        float,
        float,
        float,
        float,
        int,
        int,
        int,
        int,
        int,
        int,
        str,
    ]: ...
    def proc_open_files(pid: int, /) -> list[tuple[str, int]]: ...
    def proc_threads(pid: int, /) -> list[tuple[int, float, float]]: ...
    def proc_num_threads(pid: int, /) -> int: ...  # only FreeBSD and NetBSD
    def proc_cpu_affinity_get(pid: int, /) -> list[int]: ...  # only FreeBSD
    def proc_cpu_affinity_set(pid: int, cpu_set: Sequence[int], /) -> None: ...  # only FreeBSD
    def proc_exe(pid: int, /) -> str: ...  # only FreeBSD
    def proc_getrlimit(pid: int, resource: int, /) -> tuple[int, int]: ...  # only FreeBSD
    def proc_memory_maps(pid: int, /) -> list[tuple[str, str, str, int, int, int, int]]: ...  # only FreeBSD
    def proc_net_connections(  # only FreeBSD
        pid: int, af_filter: Sequence[AddressFamily | int | None], type_filter: Sequence[SocketKind | int | None], /
    ) -> list[tuple[int, int, int, tuple[str, int], tuple[str, int] | tuple[()], int] | tuple[int, int, int, str, str, int]]: ...
    def proc_setrlimit(pid: int, resource: int, soft: int, hard: int, /) -> None: ...  # only FreeBSD
    def boot_time() -> float: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_stats() -> tuple[int, ...]: ...  # tuple's length depends on OS
    def cpu_times() -> tuple[float, float, float, float, float]: ...
    def disk_io_counters() -> dict[str, tuple[int, ...]]: ...  # tuple's length depends on OS
    def disk_partitions() -> list[tuple[str, str, str, str]]: ...

    @overload  # for FreeBSD
    def net_connections(
        af_filter: Sequence[AddressFamily | int | None], type_filter: Sequence[SocketKind | int | None], /
    ) -> list[
        tuple[int, int, int, tuple[str, int], tuple[str, int] | tuple[()], int, int] | tuple[int, int, int, str, str, int, int]
    ]: ...
    @overload  # for OpenBSD
    def net_connections(
        pid: int, af_filter: Sequence[AddressFamily | int | None], type_filter: Sequence[SocketKind | int | None], /
    ) -> list[
        tuple[int, int, int, tuple[str, int], tuple[str, int] | tuple[()], int, int] | tuple[int, int, int, str, str, int, int]
    ]: ...
    @overload  # for NetBSD
    def net_connections(pid: int, kind: str, /) -> list[tuple[int, int, int, str, str, int, int]]: ...

    def net_io_counters() -> dict[str, tuple[int, int, int, int, int, int, int, int]]: ...
    def per_cpu_times() -> list[tuple[float, float, float, float, float]]: ...
    def pids() -> list[int]: ...
    def swap_mem() -> tuple[int, int, int, int, int]: ...
    def heap_info() -> tuple[int, int]: ...  # only FreeBSD and NetBSD
    def heap_trim() -> None: ...  # only FreeBSD and NetBSD
    def users() -> list[tuple[str, str, str, float, int | None]]: ...  # returns None only in OpenBSD
    def virtual_mem() -> tuple[int, ...]: ...  # tuple's length depends on OS

    @overload
    def cpu_freq() -> int: ...  # only OpenBSD
    @overload
    def cpu_freq(core: int, /) -> tuple[int, str]: ...  # only FreeBSD

    def cpu_topology() -> str | None: ...  # only FreeBSD
    def sensors_battery() -> tuple[int, int, int]: ...  # only FreeBSD
    def sensors_cpu_temperature(core: int, /) -> tuple[int, int]: ...  # only FreeBSD
    def check_pid_range(pid: int, /) -> None: ...
    def set_debug(value: bool, /) -> None: ...



import sys

# sys.platform.startswith(("sunos", "solaris")):
if sys.platform != "linux" and sys.platform != "win32" and sys.platform != "darwin":
    from _typeshed import Incomplete
    from collections.abc import Callable
    from typing import Final, Literal, NamedTuple, ParamSpec, TypeVar, overload

    from psutil._common import (
        AF_INET6 as AF_INET6,
        ENCODING as ENCODING,
        AccessDenied as AccessDenied,
        NoSuchProcess as NoSuchProcess,
        ZombieProcess as ZombieProcess,
        debug as debug,
        get_procfs_path as get_procfs_path,
        isfile_strict as isfile_strict,
        memoize_when_activated as memoize_when_activated,
        sockfam_to_enum as sockfam_to_enum,
        socktype_to_enum as socktype_to_enum,
        usage_percent as usage_percent,
    )

    from . import _ntuples as ntp, _psposix, _psutil_sunos

    _P = ParamSpec("_P")
    _R = TypeVar("_R")

    __extra__all__: Final[list[str]]
    PAGE_SIZE: Final[int]
    AF_LINK: Final[int]
    IS_64_BIT: Final[bool]
    CONN_IDLE: Final = "IDLE"
    CONN_BOUND: Final = "BOUND"
    PROC_STATUSES: Final[dict[int, str]]
    TCP_STATUSES: Final[dict[int, str]]
    proc_info_map: Final[dict[str, int]]

    class scputimes(NamedTuple):
        user: float
        system: float
        idle: float
        iowait: float

    class pcputimes(NamedTuple):
        user: float
        system: float
        children_user: float
        children_system: float

    class svmem(NamedTuple):
        total: int
        available: int
        percent: float
        used: int
        free: int

    class pmem(NamedTuple):
        rss: int
        vms: int

    pfullmem = pmem

    class pmmap_grouped(NamedTuple):
        path: Incomplete
        rss: Incomplete
        anonymous: Incomplete
        locked: Incomplete

    class pmmap_ext(NamedTuple):
        addr: Incomplete
        perms: Incomplete
        path: Incomplete
        rss: Incomplete
        anonymous: Incomplete
        locked: Incomplete

    def virtual_memory() -> svmem: ...
    def swap_memory() -> ntp.sswap: ...
    def cpu_times() -> scputimes: ...
    def per_cpu_times() -> list[scputimes]: ...
    def cpu_count_logical() -> int | None: ...
    def cpu_count_cores() -> int | None: ...
    def cpu_stats() -> ntp.scpustats: ...

    disk_io_counters = _psutil_sunos.disk_io_counters
    disk_usage = _psposix.disk_usage

    def disk_partitions(all: bool = False) -> list[ntp.sdiskpart]: ...

    net_io_counters = _psutil_sunos.net_io_counters
    net_if_addrs = _psutil_sunos.net_if_addrs

    @overload
    def net_connections(kind: str, _pid: Literal[-1] = -1) -> list[ntp.sconn]: ...
    @overload
    def net_connections(kind: str, _pid: int = -1) -> list[ntp.pconn]: ...

    def net_if_stats() -> dict[str, ntp.snicstats]: ...
    def boot_time() -> float: ...
    def users() -> list[ntp.suser]: ...
    def pids() -> list[int]: ...
    def pid_exists(pid: int) -> bool: ...
    def wrap_exceptions(fun: Callable[_P, _R]) -> Callable[_P, _R]: ...

    class Process:
        __slots__ = ["_cache", "_name", "_ppid", "_procfs_path", "pid"]
        pid: int
        def __init__(self, pid: int) -> None: ...
        def oneshot_enter(self) -> None: ...
        def oneshot_exit(self) -> None: ...
        def name(self) -> str: ...
        def exe(self) -> str: ...
        def cmdline(self) -> list[str] | None: ...
        def environ(self) -> dict[str, str]: ...
        def create_time(self) -> float: ...
        def num_threads(self) -> int: ...
        def nice_get(self) -> int: ...
        def nice_set(self, value: int) -> None: ...
        def ppid(self) -> int: ...
        def uids(self) -> ntp.puids: ...
        def gids(self) -> ntp.puids: ...
        def cpu_times(self) -> ntp.pcputimes: ...
        def cpu_num(self) -> int: ...
        def terminal(self) -> str | None: ...
        def cwd(self) -> str: ...
        def memory_info(self) -> pmem: ...
        memory_full_info = memory_info
        def status(self) -> str: ...
        def threads(self) -> list[ntp.pthread]: ...
        def open_files(self) -> list[ntp.popenfile]: ...
        def net_connections(self, kind: str = "inet") -> list[ntp.pconn]: ...

        class nt_mmap_grouped(NamedTuple):
            path: Incomplete
            rss: Incomplete
            anon: Incomplete
            locked: Incomplete

        class nt_mmap_ext(NamedTuple):
            addr: Incomplete
            perms: Incomplete
            path: Incomplete
            rss: Incomplete
            anon: Incomplete
            locked: Incomplete

        def memory_maps(self) -> list[tuple[str, str, str, int, int, int]]: ...
        def num_fds(self) -> int: ...
        def num_ctx_switches(self) -> ntp.pctxsw: ...
        def wait(self, timeout: float | None = None) -> int | None: ...



import ssl
from _socket import _Address
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Iterable, Mapping
from socket import AddressFamily, SocketKind
from typing import Any, TypeAlias

from ..socks import sockssocket
from ..utils.networking import HTTPHeaderDict
from .common import Request, RequestHandler, Response

def ssl_load_certs(context: ssl.SSLContext, use_certifi: bool = True) -> None: ...
def ssl_load_windows_store_certs(ssl_context: ssl.SSLContext, storename: str) -> None: ...
def make_socks_proxy_opts(socks_proxy: str) -> dict[str, Any]: ...
def get_redirect_method(method: str, status: int) -> str: ...
def make_ssl_context(
    verify: bool = True,
    client_certificate: StrOrBytesPath | None = None,
    client_certificate_key: StrOrBytesPath | None = None,
    client_certificate_password: Callable[[], str | bytes | bytearray] | str | bytes | bytearray | None = None,
    legacy_support: bool = False,
    use_certifi: bool = True,
) -> ssl.SSLContext: ...

class InstanceStoreMixin:
    def __init__(self, **kwargs: Any) -> None: ...  # Passed to non-existent parent so MRO works.

def add_accept_encoding_header(headers: HTTPHeaderDict, supported_encodings: Iterable[str]) -> None: ...
def wrap_request_errors(
    func: Callable[[RequestHandler, Request], Response | None],
) -> Callable[[RequestHandler, Request], None]: ...

_IPAddress: TypeAlias = tuple[
    AddressFamily, SocketKind, int, str, tuple[str, int] | tuple[str, int, int, int] | tuple[int, bytes]
]

def create_socks_proxy_socket(
    dest_addr: _Address, proxy_args: Mapping[str, Any], proxy_ip_addr: _IPAddress, timeout: float | None, source_address: _Address
) -> sockssocket: ...
def create_connection(
    address: tuple[str, int],
    timeout: int = ...,
    source_address: _Address | None = None,
    *,
    _create_socket_func: Callable[[_IPAddress, int, _Address], sockssocket] = ...,
) -> sockssocket: ...



from _typeshed import OptExcInfo
from collections.abc import Callable
from typing import TypeVar
from unittest import TestCase as _TestCase, TestResult

_T = TypeVar("_T")

class LocalTestResult(TestResult):
    parent_result: TestResult
    def __init__(self, parent_result: TestResult) -> None: ...
    def addError(self, test: _TestCase, error: OptExcInfo) -> None: ...
    def addFailure(self, test: _TestCase, error: OptExcInfo) -> None: ...
    def addSkip(self, test: _TestCase, reason: str) -> None: ...
    def addDuration(self, test: _TestCase, duration: float) -> None: ...

class ReferenceLeakCheckerMixin:
    NB_RUNS: int
    def run(self, result: TestResult | None = None) -> TestResult: ...

def SkipReferenceLeakChecker(reason: str) -> Callable[[_T], _T]: ...
def TestCase(test_class: _T) -> _T: ...



from _typeshed import Incomplete
from asyncio import StreamReader, StreamWriter
from socket import socket
from typing import ClassVar

class ProtocolError(Exception): ...

class ControlProtocol:
    MAX_MESSAGE_SIZE: ClassVar[int]
    @staticmethod
    def encode_message(data: dict[Incomplete, Incomplete]) -> bytes: ...
    @staticmethod
    def decode_message(data: bytes) -> dict[Incomplete, Incomplete]: ...
    @staticmethod
    def read_message(sock: socket) -> dict[Incomplete, Incomplete]: ...
    @staticmethod
    def write_message(sock: socket, data: dict[Incomplete, Incomplete]) -> None: ...
    @staticmethod
    async def read_message_async(reader: StreamReader) -> dict[Incomplete, Incomplete]: ...
    @staticmethod
    async def write_message_async(writer: StreamWriter, data: dict[Incomplete, Incomplete]) -> None: ...

# TODO: Use TypedDict for next return types
def make_request(request_id: int, command: str, args: list[str] | None = None) -> dict[str, Incomplete]: ...
def make_response(request_id: int, data: dict[Incomplete, Incomplete] | None = None) -> dict[str, Incomplete]: ...
def make_error_response(request_id: int, error: str) -> dict[str, Incomplete]: ...



from _typeshed import Incomplete
from typing import TypedDict, type_check_only

from braintree.graphql.inputs.billing_address_input import _GraphqlVariables as _BillingAddressGraphqlVariables
from braintree.graphql.inputs.shipping_address_input import _GraphqlVariables as _ShippingAddressGraphqlVariables

@type_check_only
class _GraphqlVariables(TypedDict, total=False):
    billingAddress: _BillingAddressGraphqlVariables
    email: str
    givenName: str
    phoneCountryCode: str
    phoneNumber: str
    shippingAddress: _ShippingAddressGraphqlVariables
    surname: str

class PayerInfoInput:
    def __init__(
        self,
        billing_address: dict[str, Incomplete] | None = None,
        email: str | None = None,
        given_name: str | None = None,
        phone_country_code: str | None = None,
        phone_number: str | None = None,
        shipping_address: dict[str, Incomplete] | None = None,
        surname: str | None = None,
    ) -> None: ...
    def to_graphql_variables(self) -> _GraphqlVariables: ...



from collections.abc import Callable
from enum import Enum
from typing import Any, Final, Generic, NamedTuple, NewType, TypeVar, overload

__version__: str

class MissingDependencyException(Exception): ...
class MissingDependencyError(MissingDependencyException): ...
class InvalidRegistrationException(Exception): ...
class InvalidRegistrationError(InvalidRegistrationException): ...
class InvalidForwardReferenceException(Exception): ...
class InvalidForwardReferenceError(InvalidForwardReferenceException): ...

class Scope(Enum):
    transient = 0
    singleton = 1

_T = TypeVar("_T", default=Any)

class _Registration(NamedTuple, Generic[_T]):
    service: type[_T] | str
    scope: Scope
    builder: Callable[..., _T]
    needs: dict[str, Any]  # the type hints of the builder's parameters
    args: dict[str, Any]  # passed to builder at instantiation time

_Empty = NewType("_Empty", object)  # a class at runtime
empty: Final[_Empty]

class _Registry:
    def register_service_and_impl(
        self,
        service: type[_T] | str,
        scope: Scope,
        impl: type[_T],
        resolve_args: dict[str, Any],  # forwarded to _Registration.builder
    ) -> None: ...
    def register_service_and_instance(self, service: type[_T] | str, instance: _T) -> None: ...
    def register_concrete_service(self, service: type | str, scope: Scope) -> None: ...
    def build_context(self, key: type | str, existing: _ResolutionContext | None = None) -> _ResolutionContext: ...
    def register(
        self,
        service: type[_T] | str,
        factory: Callable[..., _T] | _Empty = ...,
        instance: _T | _Empty = ...,
        scope: Scope = Scope.transient,
        **kwargs: Any,  # forwarded to _Registration.builder
    ) -> None: ...
    def __getitem__(self, service: type[_T] | str) -> list[_Registration[_T]]: ...

class _ResolutionTarget(Generic[_T]):
    service: type[_T] | str
    impls: list[_Registration[_T]]
    def __init__(self, key: type[_T] | str, impls: list[_Registration[_T]]) -> None: ...
    def is_generic_list(self) -> bool: ...
    @property
    def generic_parameter(self) -> Any: ...  # returns the first annotated generic parameter of the service
    def next_impl(self) -> _Registration[_T]: ...

class _ResolutionContext:
    targets: dict[type | str, _ResolutionTarget[Any]]
    cache: dict[type | str, Any]  # resolved objects during this resolution
    service: type | str
    def __init__(self, key: type | str, impls: list[_Registration[Any]]) -> None: ...
    def target(self, key: type[_T] | str) -> _ResolutionTarget[_T]: ...
    def has_cached(self, key: type | str) -> bool: ...
    def __getitem__(self, key: type[_T] | str) -> _T: ...
    def __setitem__(self, key: type[_T] | str, value: _T) -> None: ...
    def all_registrations(self, service: type[_T] | str) -> list[_Registration[_T]]: ...

class Container:
    registrations: _Registry
    def __init__(self) -> None: ...

    # all kwargs are forwarded to _Registration.builder
    @overload
    def register(self, service: type[_T] | str, *, instance: _T, **kwargs: Any) -> Container: ...
    @overload
    def register(
        self, service: type[_T] | str, factory: Callable[..., _T] | _Empty = ..., *, scope: Scope = Scope.transient, **kwargs: Any
    ) -> Container: ...
    @overload
    def register(
        self,
        service: type[_T] | str,
        factory: Callable[..., _T] | _Empty = ...,
        instance: _T | _Empty = ...,
        scope: Scope = Scope.transient,
        **kwargs: Any,
    ) -> Container: ...

    def resolve_all(self, service: type[_T] | str, **kwargs: Any) -> list[_T]: ...
    def resolve(self, service_key: type[_T] | str, **kwargs: Any) -> _T: ...
    def instantiate(self, service_key: type[_T] | str, **kwargs: Any) -> _T: ...



from _typeshed import Incomplete

use_ssl_context: bool

class Tls:
    ssl_options: Incomplete
    validate: Incomplete
    ca_certs_file: Incomplete
    ca_certs_path: Incomplete
    ca_certs_data: Incomplete
    private_key_password: Incomplete
    version: Incomplete
    private_key_file: Incomplete
    certificate_file: Incomplete
    valid_names: Incomplete
    ciphers: Incomplete
    sni: Incomplete
    def __init__(
        self,
        local_private_key_file=None,
        local_certificate_file=None,
        validate=...,
        version=None,
        ssl_options=None,
        ca_certs_file=None,
        valid_names=None,
        ca_certs_path=None,
        ca_certs_data=None,
        local_private_key_password=None,
        ciphers=None,
        sni=None,
    ) -> None: ...
    def wrap_socket(self, connection, do_handshake: bool = False) -> None: ...
    def start_tls(self, connection): ...

def check_hostname(sock, server_name, additional_names) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping
from ssl import SSLContext
from typing import Any, Literal, TypedDict, type_check_only
from typing_extensions import NotRequired, deprecated

import urllib3
from urllib3.connectionpool import ConnectionPool
from urllib3.contrib.socks import SOCKSProxyManager as SOCKSProxyManager
from urllib3.exceptions import (
    ConnectTimeoutError as ConnectTimeoutError,
    MaxRetryError as MaxRetryError,
    ProtocolError as ProtocolError,
    ReadTimeoutError as ReadTimeoutError,
    ResponseError as ResponseError,
)
from urllib3.poolmanager import PoolManager as PoolManager, proxy_from_url as proxy_from_url
from urllib3.util.retry import Retry as Retry

from .cookies import extract_cookies_to_jar as extract_cookies_to_jar
from .exceptions import (
    ConnectionError as ConnectionError,
    ConnectTimeout as ConnectTimeout,
    ProxyError as ProxyError,
    ReadTimeout as ReadTimeout,
    RetryError as RetryError,
    SSLError as SSLError,
)
from .models import PreparedRequest, Response as Response
from .structures import CaseInsensitiveDict as CaseInsensitiveDict
from .utils import (
    DEFAULT_CA_BUNDLE_PATH as DEFAULT_CA_BUNDLE_PATH,
    _Uri,
    get_auth_from_url as get_auth_from_url,
    get_encoding_from_headers as get_encoding_from_headers,
    prepend_scheme_if_needed as prepend_scheme_if_needed,
    urldefragauth as urldefragauth,
)

# Arguments to urllib3 connection_from_host() functions (except pool_kwargs).
@type_check_only
class _HostParams(TypedDict):
    host: str
    scheme: str
    port: int

@type_check_only
class _PoolKwargs(TypedDict):
    ssl_context: NotRequired[SSLContext]
    ca_certs: NotRequired[str]
    ca_cert_dir: NotRequired[str]
    cert_reqs: Literal["CERT_REQUIRED", "CERT_NONE"]
    cert_file: NotRequired[str]
    key_file: NotRequired[str]

DEFAULT_POOLBLOCK: bool
DEFAULT_POOLSIZE: int
DEFAULT_RETRIES: int
DEFAULT_POOL_TIMEOUT: float | None

class BaseAdapter:
    def __init__(self) -> None: ...
    def send(
        self,
        request: PreparedRequest,
        stream: bool = False,
        timeout: None | float | tuple[float, float] | tuple[float, None] = None,
        verify: bool | str = True,
        cert: None | bytes | str | tuple[bytes | str, bytes | str] = None,
        proxies: Mapping[str, str] | None = None,
    ) -> Response: ...
    def close(self) -> None: ...

class HTTPAdapter(BaseAdapter):
    __attrs__: Incomplete
    max_retries: Retry
    config: Incomplete
    proxy_manager: Incomplete
    def __init__(
        self, pool_connections: int = 10, pool_maxsize: int = 10, max_retries: Retry | int | None = 0, pool_block: bool = False
    ) -> None: ...
    poolmanager: Incomplete
    def init_poolmanager(
        self,
        connections: int,
        maxsize: int,
        block: bool = False,
        **pool_kwargs: Any,  # Any: Arbitrary keyword arguments passed directly to urllib3's PoolManager constructor.
        # Allowed types depend on urllib3 version, but typically include:
        # ssl_version (int), cert_reqs (str), ca_certs (str), ca_cert_dir (str),
        # ssl_context (ssl.SSLContext), socket_options (list), etc.
        # We use Any because the exact set is dynamic and not fully specified in stubs.
    ) -> None: ...
    def proxy_manager_for(
        self,
        proxy: str,
        **proxy_kwargs: Any,  # Any: Same as pool_kwargs above, passed to ProxyManager or SOCKSProxyManager.
        # May include: ssl_context, cert_reqs, ca_certs, ca_cert_dir, etc.
    ) -> Any:  # Any: Returns either urllib3.ProxyManager (for HTTP/HTTPS proxies) or SOCKSProxyManager (for SOCKS).
        # The exact return type depends on the proxy scheme and is not needed by callers; using Any avoids
        # circular imports or complex union types. In practice, the object adheres to a common interface.
        ...

    def cert_verify(self, conn, url, verify, cert): ...
    def build_response(self, req: PreparedRequest, resp: urllib3.BaseHTTPResponse) -> Response: ...
    def build_connection_pool_key_attributes(
        self, request: PreparedRequest, verify: bool | str, cert: str | tuple[str, str] | None = None
    ) -> tuple[_HostParams, _PoolKwargs]: ...
    def get_connection_with_tls_context(
        self,
        request: PreparedRequest,
        verify: bool | str | None,
        proxies: Mapping[str, str] | None = None,
        cert: tuple[str, str] | str | None = None,
    ) -> ConnectionPool: ...
    @deprecated("Use get_connection_with_tls_context() instead.")
    def get_connection(self, url: _Uri, proxies: Mapping[str, str] | None = None) -> ConnectionPool: ...
    def close(self) -> None: ...
    def request_url(self, request: PreparedRequest, proxies: Mapping[str, str] | None) -> str: ...
    def add_headers(
        self,
        request: PreparedRequest,
        **kwargs: Any,  # Any: Hook method for subclasses to add custom headers.
        # The kwargs mirror the send() parameters: stream (bool), timeout (float|tuple),
        # verify (bool|str), cert (str|tuple), proxies (dict). Base implementation ignores them.
        # Using Any allows subclasses to access these arguments without repeating the full signature.
    ) -> None: ...
    def proxy_headers(self, proxy: str) -> dict[str, str]: ...
    def send(
        self,
        request: PreparedRequest,
        stream: bool = False,
        timeout: None | float | tuple[float, float] | tuple[float, None] = None,
        verify: bool | str = True,
        cert: None | bytes | str | tuple[bytes | str, bytes | str] = None,
        proxies: Mapping[str, str] | None = None,
    ) -> Response: ...



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Collection, Iterable, Mapping
from typing import SupportsBytes, SupportsIndex

from authlib.jose.rfc7517 import Key, KeySet

class JsonWebKey:
    JWK_KEY_CLS: dict[Incomplete, Incomplete]
    @classmethod
    def generate_key(cls, kty, crv_or_size, options=None, is_private: bool = False): ...
    @classmethod
    def import_key(
        cls,
        raw: (
            str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer | Mapping[str, object]
        ),
        options: Mapping[str, object] | None = None,
    ) -> Key: ...
    @classmethod
    def import_key_set(cls, raw: str | Collection[str] | dict[str, object]) -> KeySet: ...



from typing import Literal
from typing_extensions import Self

from gunicorn.asgi.unreader import AsyncUnreader
from gunicorn.config import Config
from gunicorn.uwsgi.message import UWSGIRequest

from .._types import _AddressType

class AsyncUWSGIRequest(UWSGIRequest):
    cfg: Config
    unreader: AsyncUnreader  # type: ignore[assignment]
    peer_addr: _AddressType
    remote_addr: _AddressType
    req_number: int
    method: str | None
    uri: str | None
    path: str | None
    query: str | None
    fragment: str | None
    version: tuple[int, int]
    headers: list[tuple[str, str]]
    trailers: list[tuple[str, str]]
    scheme: Literal["https", "http"]
    must_close: bool
    uwsgi_vars: dict[str, str]
    modifier1: int
    modifier2: int
    proxy_protocol_info: dict[str, str | int | None] | None  # TODO: Use TypedDict
    content_length: int
    chunked: bool

    def __init__(self, cfg: Config, unreader: AsyncUnreader, peer_addr: _AddressType, req_number: int = 1) -> None: ...
    @classmethod
    async def parse(cls, cfg: Config, unreader: AsyncUnreader, peer_addr: _AddressType, req_number: int = 1) -> Self: ...  # type: ignore[override]
    async def read_body(self, size: int = 8192) -> bytes: ...
    async def drain_body(self) -> None: ...
    def get_header(self, name: str) -> str | None: ...



from collections.abc import Iterable
from typing import NamedTuple

from .ast import Node

class RGBA(NamedTuple):
    red: float
    green: float
    blue: float
    alpha: float

def parse_color(input: str | Iterable[Node]) -> str | RGBA | None: ...



from _typeshed import Incomplete

class PersistentSearch:
    connection: Incomplete
    changes_only: Incomplete
    notifications: Incomplete
    message_id: Incomplete
    base: Incomplete
    filter: Incomplete
    scope: Incomplete
    dereference_aliases: Incomplete
    attributes: Incomplete
    size_limit: Incomplete
    time_limit: Incomplete
    controls: Incomplete
    def __init__(
        self,
        connection,
        search_base,
        search_filter,
        search_scope,
        dereference_aliases,
        attributes,
        size_limit,
        time_limit,
        controls,
        changes_only,
        events_type,
        notifications,
        streaming,
        callback,
    ) -> None: ...
    def start(self) -> None: ...
    def stop(self, unbind: bool = True) -> None: ...
    def next(self, block: bool = False, timeout=None): ...
    def funnel(self, block: bool = False, timeout=None) -> None: ...



from _typeshed import Incomplete

signals_available: bool

class Namespace:
    def signal(self, name: str, doc: str | None = None) -> _FakeSignal: ...

class _FakeSignal:
    name: str
    __doc__: str | None
    def __init__(self, name: str, doc: str | None = None) -> None: ...
    send: Incomplete
    connect: Incomplete
    disconnect: Incomplete
    has_receivers_for: Incomplete
    receivers_for: Incomplete
    temporarily_connected_to: Incomplete
    connected_to: Incomplete

scope_changed: _FakeSignal



from typing import Final

class TransactionAmounts:
    Authorize: Final = "1000.00"
    PartiallyAuthorized: Final = "1004.00"
    Decline: Final = "2000.00"
    HardDecline: Final = "2015.00"
    Fail: Final = "3000.00"



from typing import Any, Final, final, overload
from typing_extensions import Self

__version__: Final[str]
ATTR_CASE: Final = 3271982
CASE_LOWER: Final = 1
CASE_NATURAL: Final = 0
CASE_UPPER: Final = 2
PARAM_FILE: Final = 11
QUOTED_LITERAL_REPLACEMENT_OFF: Final = 0
QUOTED_LITERAL_REPLACEMENT_ON: Final = 1
SQL_API_SQLROWCOUNT: Final[int]
SQL_ATTR_AUTOCOMMIT: Final[int]
SQL_ATTR_CALL_RETURN: Final[int]
SQL_ATTR_CURRENT_SCHEMA: Final[int]
SQL_ATTR_CURSOR_TYPE: Final[int]
SQL_ATTR_INFO_ACCTSTR: Final[int]
SQL_ATTR_INFO_APPLNAME: Final[int]
SQL_ATTR_INFO_PROGRAMNAME: Final[int]
SQL_ATTR_INFO_USERID: Final[int]
SQL_ATTR_INFO_WRKSTNNAME: Final[int]
SQL_ATTR_PARAMSET_SIZE: Final[int]
SQL_ATTR_PARAM_BIND_TYPE: Final[int]
SQL_ATTR_QUERY_TIMEOUT: Final[int]
SQL_ATTR_ROWCOUNT_PREFETCH: Final[int]
SQL_ATTR_TRUSTED_CONTEXT_PASSWORD: Final[int]
SQL_ATTR_TRUSTED_CONTEXT_USERID: Final[int]
SQL_ATTR_TXN_ISOLATION: Final[int]
SQL_ATTR_USE_TRUSTED_CONTEXT: Final[int]
SQL_ATTR_XML_DECLARATION: Final[int]
SQL_AUTOCOMMIT_OFF: Final[int]
SQL_AUTOCOMMIT_ON: Final[int]
SQL_BIGINT: Final[int]
SQL_BINARY: Final[int]
SQL_BIT: Final[int]
SQL_BLOB: Final[int]
SQL_BLOB_LOCATOR: Final[int]
SQL_BOOLEAN: Final[int]
SQL_CHAR: Final[int]
SQL_CLOB: Final[int]
SQL_CLOB_LOCATOR: Final[int]
SQL_CURSOR_DYNAMIC: Final[int]
SQL_CURSOR_FORWARD_ONLY: Final[int]
SQL_CURSOR_KEYSET_DRIVEN: Final[int]
SQL_CURSOR_STATIC: Final[int]
SQL_DBCLOB: Final[int]
SQL_DBCLOB_LOCATOR: Final[int]
SQL_DBMS_NAME: Final[int]
SQL_DBMS_VER: Final[int]
SQL_DECFLOAT: Final[int]
SQL_DECIMAL: Final[int]
SQL_DOUBLE: Final[int]
SQL_FALSE: Final[int]
SQL_FLOAT: Final[int]
SQL_GRAPHIC: Final[int]
SQL_INDEX_CLUSTERED: Final[int]
SQL_INDEX_OTHER: Final[int]
SQL_INTEGER: Final[int]
SQL_LONGVARBINARY: Final[int]
SQL_LONGVARCHAR: Final[int]
SQL_LONGVARGRAPHIC: Final[int]
SQL_NUMERIC: Final[int]
SQL_PARAM_BIND_BY_COLUMN: Final[int]
SQL_PARAM_INPUT: Final[int]
SQL_PARAM_INPUT_OUTPUT: Final[int]
SQL_PARAM_OUTPUT: Final[int]
SQL_REAL: Final[int]
SQL_ROWCOUNT_PREFETCH_OFF: Final[int]
SQL_ROWCOUNT_PREFETCH_ON: Final[int]
SQL_SMALLINT: Final[int]
SQL_TABLE_STAT: Final[int]
SQL_TINYINT: Final[int]
SQL_TRUE: Final[int]
SQL_TXN_NO_COMMIT: Final[int]
SQL_TXN_READ_COMMITTED: Final[int]
SQL_TXN_READ_UNCOMMITTED: Final[int]
SQL_TXN_REPEATABLE_READ: Final[int]
SQL_TXN_SERIALIZABLE: Final[int]
SQL_TYPE_DATE: Final[int]
SQL_TYPE_TIME: Final[int]
SQL_TYPE_TIMESTAMP: Final[int]
SQL_VARBINARY: Final[int]
SQL_VARCHAR: Final[int]
SQL_VARGRAPHIC: Final[int]
SQL_WCHAR: Final[int]
SQL_WLONGVARCHAR: Final[int]
SQL_WVARCHAR: Final[int]
SQL_XML: Final[int]
USE_WCHAR: Final = 100
WCHAR_NO: Final = 0
WCHAR_YES: Final = 1

SQL_ATTR_ACCESS_MODE: Final[int]
SQL_ATTR_ALLOW_INTERLEAVED_GETDATA: Final[int]
SQL_ATTR_ANSI_APP: Final[int]
SQL_ATTR_APPEND_FOR_FETCH_ONLY: Final[int]
SQL_ATTR_APP_USES_LOB_LOCATOR: Final[int]
SQL_ATTR_ASYNC_ENABLE: Final[int]
SQL_ATTR_AUTO_IPD: Final[int]
SQL_ATTR_CACHE_USRLIBL: Final[int]
SQL_ATTR_CLIENT_APPLCOMPAT: Final[int]
SQL_ATTR_CLIENT_CODEPAGE: Final[int]
SQL_ATTR_COLUMNWISE_MRI: Final[int]
SQL_ATTR_COMMITONEOF: Final[int]
SQL_ATTR_CONCURRENT_ACCESS_RESOLUTION: Final[int]
SQL_ATTR_CONFIG_KEYWORDS_ARRAY_SIZE: Final[int]
SQL_ATTR_CONFIG_KEYWORDS_MAXLEN: Final[int]
SQL_ATTR_CONNECTION_DEAD: Final[int]
SQL_ATTR_CONNECTTYPE: Final[int]
SQL_ATTR_CONNECT_NODE: Final[int]
SQL_ATTR_CONNECT_PASSIVE: Final[int]
SQL_ATTR_CONN_CONTEXT: Final[int]
SQL_ATTR_CURRENT_CATALOG: Final[int]
SQL_ATTR_CURRENT_IMPLICIT_XMLPARSE_OPTION: Final[int]
SQL_ATTR_CURRENT_PACKAGE_PATH: Final[int]
SQL_ATTR_CURRENT_PACKAGE_SET: Final[int]
SQL_ATTR_DATE_FMT: Final[int]
SQL_ATTR_DATE_SEP: Final[int]
SQL_ATTR_DB2EXPLAIN: Final[int]
SQL_ATTR_DB2_APPLICATION_HANDLE: Final[int]
SQL_ATTR_DB2_APPLICATION_ID: Final[int]
SQL_ATTR_DB2_SQLERRP: Final[int]
SQL_ATTR_DECFLOAT_ROUNDING_MODE: Final[int]
SQL_ATTR_DECIMAL_SEP: Final[int]
SQL_ATTR_DESCRIBE_CALL: Final[int]
SQL_ATTR_DESCRIBE_OUTPUT_LEVEL: Final[int]
SQL_ATTR_DETECT_READ_ONLY_TXN: Final[int]
SQL_ATTR_ENLIST_IN_DTC: Final[int]
SQL_ATTR_EXTENDED_INDICATORS: Final[int]
SQL_ATTR_FET_BUF_SIZE: Final[int]
SQL_ATTR_FORCE_ROLLBACK: Final[int]
SQL_ATTR_FREE_LOCATORS_ON_FETCH: Final[int]
SQL_ATTR_GET_LATEST_MEMBER: Final[int]
SQL_ATTR_GET_LATEST_MEMBER_NAME: Final[int]
SQL_ATTR_IGNORE_SERVER_LIST: Final[int]
SQL_ATTR_INFO_CRRTKN: Final[int]
SQL_ATTR_INFO_PROGRAMID: Final[int]
SQL_ATTR_KEEP_DYNAMIC: Final[int]
SQL_ATTR_LOB_CACHE_SIZE: Final[int]
SQL_ATTR_LOB_FILE_THRESHOLD: Final[int]
SQL_ATTR_LOGIN_TIMEOUT: Final[int]
SQL_ATTR_LONGDATA_COMPAT: Final[int]
SQL_ATTR_MAPCHAR: Final[int]
SQL_ATTR_MAXBLKEXT: Final[int]
SQL_ATTR_MAX_LOB_BLOCK_SIZE: Final[int]
SQL_ATTR_NETWORK_STATISTICS: Final[int]
SQL_ATTR_OVERRIDE_CHARACTER_CODEPAGE: Final[int]
SQL_ATTR_OVERRIDE_CODEPAGE: Final[int]
SQL_ATTR_OVERRIDE_PRIMARY_AFFINITY: Final[int]
SQL_ATTR_PARC_BATCH: Final[int]
SQL_ATTR_PING_DB: Final[int]
SQL_ATTR_PING_NTIMES: Final[int]
SQL_ATTR_PING_REQUEST_PACKET_SIZE: Final[int]
SQL_ATTR_QUERY_PREFETCH: Final[int]
SQL_ATTR_QUIET_MODE: Final[int]
SQL_ATTR_READ_ONLY_CONNECTION: Final[int]
SQL_ATTR_RECEIVE_TIMEOUT: Final[int]
SQL_ATTR_REOPT: Final[int]
SQL_ATTR_REPORT_ISLONG_FOR_LONGTYPES_OLEDB: Final[int]
SQL_ATTR_REPORT_SEAMLESSFAILOVER_WARNING: Final[int]
SQL_ATTR_REPORT_TIMESTAMP_TRUNC_AS_WARN: Final[int]
SQL_ATTR_RETRYONERROR: Final[int]
SQL_ATTR_RETRY_ON_MERGE: Final[int]
SQL_ATTR_SERVER_MSGTXT_MASK: Final[int]
SQL_ATTR_SERVER_MSGTXT_SP: Final[int]
SQL_ATTR_SESSION_GLOBAL_VAR: Final[int]
SQL_ATTR_SESSION_TIME_ZONE: Final[int]
SQL_ATTR_SPECIAL_REGISTER: Final[int]
SQL_ATTR_SQLCOLUMNS_SORT_BY_ORDINAL_OLEDB: Final[int]
SQL_ATTR_STMT_CONCENTRATOR: Final[int]
SQL_ATTR_STREAM_GETDATA: Final[int]
SQL_ATTR_STREAM_OUTPUTLOB_ON_CALL: Final[int]
SQL_ATTR_TIME_FMT: Final[int]
SQL_ATTR_TIME_SEP: Final[int]
SQL_ATTR_TRUSTED_CONTEXT_ACCESSTOKEN: Final[int]
SQL_ATTR_USER_REGISTRY_NAME: Final[int]
SQL_ATTR_WCHARTYPE: Final[int]

@final
class IBM_DBClientInfo:
    def __new__(cls, *args: object, **kwargs: object) -> Self: ...
    APPL_CODEPAGE: int
    CONN_CODEPAGE: int
    DATA_SOURCE_NAME: str
    DRIVER_NAME: str
    DRIVER_ODBC_VER: str
    DRIVER_VER: str
    ODBC_SQL_CONFORMANCE: str
    ODBC_VER: str

@final
class IBM_DBConnection:
    def __new__(cls, *args: object, **kwargs: object) -> Self: ...

@final
class IBM_DBServerInfo:
    def __new__(cls, *args: object, **kwargs: object) -> Self: ...
    DBMS_NAME: str
    DBMS_VER: str
    DB_CODEPAGE: int
    DB_NAME: str
    DFT_ISOLATION: str
    IDENTIFIER_QUOTE_CHAR: str
    INST_NAME: str
    ISOLATION_OPTION: tuple[str, str, str, str, str]
    KEYWORDS: str
    LIKE_ESCAPE_CLAUSE: bool
    MAX_COL_NAME_LEN: int
    MAX_IDENTIFIER_LEN: int
    MAX_INDEX_SIZE: int
    MAX_PROC_NAME_LEN: int
    MAX_ROW_SIZE: int
    MAX_SCHEMA_NAME_LEN: int
    MAX_STATEMENT_LEN: int
    MAX_TABLE_NAME_LEN: int
    NON_NULLABLE_COLUMNS: bool
    PROCEDURES: bool
    SPECIAL_CHARS: str
    SQL_CONFORMANCE: str

@final
class IBM_DBStatement:
    def __new__(cls, *args: object, **kwargs: object) -> Self: ...

def active(connection: IBM_DBConnection | None, /) -> bool: ...
def autocommit(connection: IBM_DBConnection, value: int = ..., /) -> int | bool: ...
def bind_param(
    stmt: IBM_DBStatement,
    parameter_number: int,
    variable: str,
    parameter_type: int | None = ...,
    data_type: int | None = ...,
    precision: int | None = ...,
    scale: int | None = ...,
    size: int | None = ...,
    /,
) -> bool: ...

@overload
def callproc(connection: IBM_DBConnection, procname: str, /) -> IBM_DBStatement | None: ...
@overload
def callproc(connection: IBM_DBConnection, procname: str, parameters: tuple[object, ...], /) -> tuple[object, ...] | None: ...

def check_function_support(connection: IBM_DBConnection, function_id: int, /) -> bool: ...
def client_info(connection: IBM_DBConnection, /) -> IBM_DBClientInfo | bool: ...
def close(connection: IBM_DBConnection, /) -> bool: ...
def column_privileges(
    connection: IBM_DBConnection,
    qualifier: str | None = ...,
    schema: str | None = ...,
    table_name: str | None = ...,
    column_name: str | None = ...,
    /,
) -> IBM_DBStatement: ...
def columns(
    connection: IBM_DBConnection,
    qualifier: str | None = ...,
    schema: str | None = ...,
    table_name: str | None = ...,
    column_name: str | None = ...,
    /,
) -> IBM_DBStatement: ...
def commit(connection: IBM_DBConnection, /) -> bool: ...
def conn_error(connection: IBM_DBConnection | None = ..., /) -> str: ...
def conn_errormsg(connection: IBM_DBConnection | None = ..., /) -> str: ...
def conn_warn(connection: IBM_DBConnection | None = ..., /) -> str: ...
def connect(
    database: str, user: str, password: str, options: dict[int, int | str] | None = ..., replace_quoted_literal: int = ..., /
) -> IBM_DBConnection | None: ...
def createdb(connection: IBM_DBConnection, dbName: str, codeSet: str = ..., mode: str = ..., /) -> bool: ...
def createdbNX(connection: IBM_DBConnection, dbName: str, codeSet: str = ..., mode: str = ..., /) -> bool: ...
def cursor_type(stmt: IBM_DBStatement, /) -> int: ...
def debug(option: str | bool) -> None: ...
def dropdb(connection: IBM_DBConnection, dbName: str, /) -> bool: ...
def exec_immediate(
    connection: IBM_DBConnection, statement: str | None, options: dict[int, int] = ..., /
) -> IBM_DBStatement | bool: ...
def execute(stmt: IBM_DBStatement, parameters: tuple[object, ...] | None = ..., /) -> bool: ...
def execute_many(
    stmt: IBM_DBStatement, seq_of_parameters: tuple[object, ...], options: dict[int, int] = ..., /
) -> int | None: ...
def fetchall(stmt: IBM_DBStatement, /) -> list[tuple[object, ...]]: ...
def fetchmany(stmt: IBM_DBStatement, numberOfRows: int, /) -> list[tuple[object, ...]]: ...
def fetchone(stmt: IBM_DBStatement, /) -> tuple[object, ...]: ...
def fetch_assoc(stmt: IBM_DBStatement, row_number: int = ..., /) -> dict[str, object] | bool: ...
def fetch_both(stmt: IBM_DBStatement, row_number: int = ..., /) -> dict[int | str, object] | bool: ...
def fetch_row(stmt: IBM_DBStatement, row_number: int = ..., /) -> bool: ...
def fetch_tuple(stmt: IBM_DBStatement, row_number: int = ..., /) -> tuple[object, ...]: ...
def field_display_size(stmt: IBM_DBStatement, column: int | str, /) -> int | bool: ...
def field_name(stmt: IBM_DBStatement, column: int | str, /) -> str | bool: ...
def field_nullable(stmt: IBM_DBStatement, column: int | str, /) -> bool: ...
def field_num(stmt: IBM_DBStatement, column: int | str, /) -> int | bool: ...
def field_precision(stmt: IBM_DBStatement, column: int | str, /) -> int | bool: ...
def field_scale(stmt: IBM_DBStatement, column: int | str, /) -> int | bool: ...
def field_type(stmt: IBM_DBStatement, column: int | str, /) -> str | bool: ...
def field_width(stmt: IBM_DBStatement, column: int | str, /) -> int | bool: ...
def foreign_keys(
    connection: IBM_DBConnection,
    pk_qualifier: str | None,
    pk_schema: str | None,
    pk_table_name: str | None,
    fk_qualifier: str | None = ...,
    fk_schema: str | None = ...,
    fk_table_name: str | None = ...,
    /,
) -> IBM_DBStatement: ...
def free_result(stmt: IBM_DBStatement, /) -> bool: ...
def free_stmt(stmt: IBM_DBStatement, /) -> bool: ...
def get_db_info(connection: IBM_DBConnection, option: int, /) -> str | bool: ...
def get_last_serial_value(stmt: IBM_DBStatement, /) -> str | bool: ...
def get_num_result(stmt: IBM_DBStatement, /) -> int | bool: ...
def get_option(resc: IBM_DBConnection | IBM_DBStatement, options: int, type: int, /) -> Any: ...
def get_sqlcode(connection_or_stmt: IBM_DBConnection | IBM_DBStatement | None = None, /) -> str: ...
def next_result(stmt: IBM_DBStatement, /) -> IBM_DBStatement | bool: ...
def num_fields(stmt: IBM_DBStatement, /) -> int | bool: ...
def num_rows(stmt: IBM_DBStatement, /) -> int: ...
def pconnect(
    database: str, username: str, password: str, options: dict[int, int | str] | None = ..., /
) -> IBM_DBConnection | None: ...
def prepare(
    connection: IBM_DBConnection, statement: str, options: dict[int, int | str] | None = ..., /
) -> IBM_DBStatement | bool: ...
def primary_keys(
    connection: IBM_DBConnection, qualifier: str | None, schema: str | None, table_name: str | None, /
) -> IBM_DBStatement: ...
def procedure_columns(
    connection: IBM_DBConnection, qualifier: str | None, schema: str | None, procedure: str | None, parameter: str | None, /
) -> IBM_DBStatement | bool: ...
def procedures(
    connection: IBM_DBConnection, qualifier: str | None, schema: str | None, procedure: str | None, /
) -> IBM_DBStatement | bool: ...
def recreatedb(connection: IBM_DBConnection, dbName: str, codeSet: str | None = ..., mode: str | None = ..., /) -> bool: ...
def result(stmt: IBM_DBStatement, column: int | str, /) -> Any: ...
def rollback(connection: IBM_DBConnection, /) -> bool: ...
def server_info(connection: IBM_DBConnection, /) -> IBM_DBServerInfo | bool: ...
def set_option(resc: IBM_DBConnection | IBM_DBStatement, options: dict[int, int | str], type: int, /) -> bool: ...
def special_columns(
    connection: IBM_DBConnection, qualifier: str | None, schema: str | None, table_name: str | None, scope: int, /
) -> IBM_DBStatement: ...
def statistics(
    connection: IBM_DBConnection, qualifier: str | None, schema: str | None, table_name: str | None, unique: bool | None, /
) -> IBM_DBStatement: ...
def stmt_error(stmt: IBM_DBStatement = ..., /) -> str: ...
def stmt_errormsg(stmt: IBM_DBStatement = ..., /) -> str: ...
def stmt_warn(connection: IBM_DBConnection = ..., /) -> IBM_DBStatement: ...
def table_privileges(
    connection: IBM_DBConnection, qualifier: str | None = ..., schema: str | None = ..., table_name: str | None = ..., /
) -> IBM_DBStatement | bool: ...
def tables(
    connection: IBM_DBConnection,
    qualifier: str | None = ...,
    schema: str | None = ...,
    table_name: str | None = ...,
    table_type: str | None = ...,
    /,
) -> IBM_DBStatement | bool: ...



from collections.abc import Mapping
from typing import Any, Final

__version__: Final[str]

def dictformat(
    _format: str, L: Mapping[str, object] | None = {}, G: dict[str, Any] | None = {}  # `L` and `G` are passed to `eval` function
) -> str: ...
def magicformat(format: str) -> str: ...



from _typeshed import Incomplete
from typing import Final

HTTPX_CLIENT_KWARGS: Final[list[str]]

def extract_client_kwargs(kwargs) -> dict[str, Incomplete]: ...
def build_request(url, headers, body, initial_request): ...



from collections.abc import Iterable
from typing import TypeAlias

from .ast import AtRule, Comment, Declaration, Node, ParseError, QualifiedRule, WhitespaceToken

_Rule: TypeAlias = QualifiedRule | AtRule | Comment | WhitespaceToken | ParseError

def parse_one_component_value(input: str | Iterable[Node], skip_comments: bool = False) -> Node: ...
def parse_one_declaration(input: str | Iterable[Node], skip_comments: bool = False) -> Declaration | ParseError: ...
def parse_blocks_contents(
    input: str | Iterable[Node], skip_comments: bool = False, skip_whitespace: bool = False
) -> list[Declaration | AtRule | QualifiedRule | Comment | WhitespaceToken | ParseError]: ...
def parse_declaration_list(
    input: str | Iterable[Node], skip_comments: bool = False, skip_whitespace: bool = False
) -> list[Declaration | AtRule | Comment | WhitespaceToken | ParseError]: ...
def parse_one_rule(input: str | Iterable[Node], skip_comments: bool = False) -> QualifiedRule | AtRule | ParseError: ...
def parse_rule_list(input: str | Iterable[Node], skip_comments: bool = False, skip_whitespace: bool = False) -> list[_Rule]: ...
def parse_stylesheet(input: str | Iterable[Node], skip_comments: bool = False, skip_whitespace: bool = False) -> list[_Rule]: ...



import datetime
from _typeshed import ConvertibleToFloat, ConvertibleToInt, Incomplete, SupportsKeysAndGetItem, Unused
from collections.abc import Iterable, Iterator
from enum import Enum
from re import Pattern
from typing import Any, ClassVar, Final, Literal, Protocol, SupportsIndex, TypeAlias, overload, type_check_only
from typing_extensions import Self

from .caselessdict import CaselessDict
from .parser import Parameters
from .parser_tools import ICAL_TYPE
from .timezone import tzid_from_dt as tzid_from_dt, tzid_from_tzinfo as tzid_from_tzinfo

__all__ = [
    "DURATION_REGEX",
    "TimeBase",
    "TypesFactory",
    "WEEKDAY_RULE",
    "tzid_from_dt",
    "vBinary",
    "vBoolean",
    "vCalAddress",
    "vCategory",
    "vDDDLists",
    "vDDDTypes",
    "vDate",
    "vDatetime",
    "vDuration",
    "vFloat",
    "vFrequency",
    "vGeo",
    "vInline",
    "vInt",
    "vMonth",
    "vPeriod",
    "vRecur",
    "vText",
    "vTime",
    "vUTCOffset",
    "vUri",
    "vWeekday",
    "tzid_from_tzinfo",
    "vSkip",
]

_PropType: TypeAlias = type[Any]  # any of the v* classes in this file
_PeriodTuple: TypeAlias = tuple[datetime.datetime, datetime.datetime | datetime.timedelta]
_AnyTimeType: TypeAlias = datetime.datetime | datetime.date | datetime.timedelta | datetime.time | _PeriodTuple

@type_check_only
class _vType(Protocol):
    def to_ical(self) -> bytes | str: ...

DURATION_REGEX: Final[Pattern[str]]
WEEKDAY_RULE: Final[Pattern[str]]

class vBinary:
    obj: str
    params: Parameters
    def __init__(self, obj: str | bytes) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> bytes: ...
    def __eq__(self, other: object) -> bool: ...

class vBoolean(int):
    BOOL_MAP: Final[CaselessDict[bool]]
    params: Parameters
    def __new__(cls, x: ConvertibleToInt = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> Literal[b"TRUE", b"FALSE"]: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> bool: ...

class vText(str):
    __slots__ = ("encoding", "params")
    encoding: str
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...
    ALTREP: property
    LANGUAGE: property
    RELTYPE: property

class vCalAddress(str):
    __slots__ = ("params",)
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...
    @property
    def email(self) -> str: ...

    @property
    def name(self) -> str: ...
    @name.setter
    def name(self, value: str) -> None: ...
    @name.deleter
    def name(self) -> None: ...

    CN: property
    CUTYPE: property
    DELEGATED_FROM: property
    DELEGATED_TO: property
    DIR: property
    LANGUAGE: property
    PARTSTAT: property
    ROLE: property
    RSVP: property
    SENT_BY: property

class vFloat(float):
    params: Parameters
    def __new__(cls, x: ConvertibleToFloat = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vInt(int):
    params: Parameters
    def __new__(cls, x: ConvertibleToInt = ..., /, *, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vDDDLists:
    params: Parameters
    dts: list[vDDDTypes]
    def __init__(self, dt_list: Iterable[_AnyTimeType] | _AnyTimeType) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: str, timezone: str | datetime.timezone | None = None) -> list[Incomplete]: ...
    def __eq__(self, other: object) -> bool: ...

class vCategory:
    cats: list[vText]
    params: Parameters
    def __init__(self, c_list: Iterable[ICAL_TYPE] | ICAL_TYPE, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> str: ...
    def __eq__(self, other: object) -> bool: ...
    RANGE: property
    RELATED: property
    TZID: property

class TimeBase:
    params: Parameters
    ignore_for_equality: set[str]
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...
    RANGE: property
    RELATED: property
    TZID: property

class vDDDTypes(TimeBase):
    params: Parameters
    dt: _AnyTimeType
    def __init__(self, dt: _AnyTimeType) -> None: ...
    def to_ical(self) -> bytes: ...

    @overload
    @classmethod
    def from_ical(cls, ical: Self, timezone: Unused | None = None) -> _AnyTimeType: ...
    # Return type is one of vDuration, vPeriod, vDatetime, vDate, or vTime,
    # depending on the ical string.
    @overload
    @classmethod
    def from_ical(cls, ical: str, timezone: datetime.timezone | str | None = None) -> Any: ...

class vDate(TimeBase):
    dt: datetime.date
    params: Parameters
    def __init__(self, dt: datetime.date) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> datetime.date: ...

class vDatetime(TimeBase):
    dt: datetime.datetime
    params: Parameters
    def __init__(self, dt: datetime.datetime, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE, timezone: datetime.timezone | str | None = None) -> datetime.datetime: ...

class vDuration(TimeBase):
    td: datetime.timedelta
    params: Parameters
    def __init__(self, td: datetime.timedelta, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> bytes: ...
    @staticmethod
    def from_ical(ical: str) -> datetime.timedelta: ...
    @property
    def dt(self) -> datetime.timedelta: ...

class vPeriod(TimeBase):
    params: Parameters
    start: datetime.datetime
    end: datetime.datetime
    by_duration: bool
    duration: datetime.timedelta
    def __init__(self, per: _PeriodTuple) -> None: ...
    def overlaps(self, other: vPeriod) -> bool: ...
    def to_ical(self) -> bytes: ...
    # Return type is a tuple of vDuration, vPeriod, vDatetime, vDate, or vTime,
    # depending on the ical string. If the ical string is formed according to
    # the iCalendar specification, this should always return a
    # (datetime, datetime) or a (datetime, timedelta) tuple, but this is not
    # enforced.
    @staticmethod
    def from_ical(ical: str, timezone: datetime.timezone | str | None = None) -> tuple[Any, Any]: ...
    @property
    def dt(self) -> _PeriodTuple: ...
    FBTYPE: property

class vWeekday(str):
    __slots__ = ("params", "relative", "weekday")
    week_days: Final[CaselessDict[int]]
    weekday: Literal["SU", "MO", "TU", "WE", "TH", "FR", "SA"] | None
    relative: int | None
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vFrequency(str):
    __slots__ = ("params",)
    frequencies: Final[CaselessDict[str]]
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vMonth(int):
    params: Parameters
    def __new__(cls, month: vMonth | str | int, params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: vMonth | str | int) -> Self: ...

    @property
    def leap(self) -> bool: ...
    @leap.setter
    def leap(self, value: bool) -> None: ...

class vSkip(vText, Enum):
    OMIT = "OMIT"
    FORWARD = "FORWARD"
    BACKWARD = "BACKWARD"

    def __reduce_ex__(self, _p: Unused) -> tuple[Self, tuple[str]]: ...

# The type of the values depend on the key. Each key maps to a v* class, and
# the allowed types are the types that the corresponding v* class can parse.
class vRecur(CaselessDict[Iterable[Any] | Any]):
    params: Parameters
    frequencies: Final[list[str]]
    canonical_order: ClassVar[tuple[str, ...]]
    types: Final[CaselessDict[_PropType]]
    def __init__(
        self, *args, params: SupportsKeysAndGetItem[str, str] = {}, **kwargs: list[Any] | tuple[Any, ...] | Any
    ) -> None: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def parse_type(cls, key: str, values: str) -> list[Any]: ...  # Returns a list of v* objects
    @classmethod
    def from_ical(cls, ical: vRecur | str) -> Self: ...

class vTime(TimeBase):
    dt: datetime.time | datetime.datetime
    params: Parameters

    @overload
    def __init__(self, dt: datetime.time | datetime.datetime, /) -> None: ...
    # args are passed to the datetime.time() constructor
    @overload
    def __init__(
        self,
        hour: SupportsIndex = ...,
        minute: SupportsIndex = ...,
        second: SupportsIndex = ...,
        microsecond: SupportsIndex = ...,
        tzinfo: datetime.tzinfo | None = ...,
        /,
    ) -> None: ...

    def to_ical(self) -> str: ...
    @staticmethod
    def from_ical(ical: ICAL_TYPE) -> datetime.time: ...

class vUri(str):
    __slots__ = ("params",)
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class vGeo:
    latitude: float
    longitude: float
    params: Parameters
    def __init__(self, geo: tuple[float | str, float | str], params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> str: ...
    @staticmethod
    def from_ical(ical: str) -> tuple[float, float]: ...
    def __eq__(self, other: _vType) -> bool: ...  # type: ignore[override]

class vUTCOffset:
    ignore_exceptions: bool
    td: datetime.timedelta
    params: Parameters
    def __init__(self, td: datetime.timedelta, params: SupportsKeysAndGetItem[str, str] = {}) -> None: ...
    def to_ical(self) -> str: ...
    @classmethod
    def from_ical(cls, ical: Self | ICAL_TYPE) -> datetime.timedelta: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...

class vInline(str):
    __slots__ = ("params",)
    params: Parameters
    def __new__(cls, value: ICAL_TYPE, encoding: str = "utf-8", params: SupportsKeysAndGetItem[str, str] = {}) -> Self: ...
    def to_ical(self) -> bytes: ...
    @classmethod
    def from_ical(cls, ical: ICAL_TYPE) -> Self: ...

class TypesFactory(CaselessDict[_PropType]):
    all_types: tuple[_PropType, ...]
    types_map: CaselessDict[str]
    def for_property(self, name: str) -> _PropType: ...
    # value is str | bytes, depending on what the v* class supports
    def to_ical(self, name: str, value: Any) -> bytes: ...
    # value and return type depend on what the v* class supports
    def from_ical(self, name: str, value: Any) -> Any: ...



import re
from _typeshed import Incomplete
from itertools import chain
from typing import Final, overload

from dateparser.conf import Settings

PARSER_HARDCODED_TOKENS: Final[list[str]]
PARSER_KNOWN_TOKENS: Final[list[str]]
ALWAYS_KEEP_TOKENS: Final[list[str]]
KNOWN_WORD_TOKENS: Final[list[str]]
PARENTHESES_PATTERN: Final[re.Pattern[str]]
NUMERAL_PATTERN: Final[re.Pattern[str]]
KEEP_TOKEN_PATTERN: Final[re.Pattern[str]]

class UnknownTokenError(Exception): ...

class Dictionary:
    info: dict[str, Incomplete]
    def __init__(self, locale_info: dict[str, Incomplete], settings: Settings | None = None) -> None: ...
    def __contains__(self, key: str) -> bool: ...
    def __getitem__(self, key: str): ...
    def __iter__(self) -> chain[str]: ...
    def are_tokens_valid(self, tokens: list[str]) -> bool: ...

    @overload
    def split(self, string: None, keep_formatting: bool = False) -> None: ...
    @overload
    def split(self, string: str, keep_formatting: bool = False) -> list[str]: ...

class NormalizedDictionary(Dictionary):
    def __init__(self, locale_info: dict[str, Incomplete], settings: Settings | None = None) -> None: ...



from types import ModuleType
from typing import Any

from google.protobuf.descriptor import FileDescriptor

def BuildMessageAndEnumDescriptors(file_des: FileDescriptor, module: dict[str, Any]) -> None: ...
def BuildTopDescriptorsAndMessages(file_des: FileDescriptor, module_name: str, module: dict[str, Any]) -> None: ...
def AddHelpersToExtensions(file_des: FileDescriptor) -> None: ...
def BuildServices(file_des: FileDescriptor, module_name: str, module: ModuleType) -> None: ...



from _typeshed import Incomplete

class Options:
    __slots__ = (
        "retries",
        "timeout",
        "use_cache",
        "use_global_cache",
        "global_cache_timeout",
        "use_datastore",
        "force_writes",
        "max_memcache_items",
        "propagation",
        "deadline",
        "use_memcache",
        "memcache_timeout",
    )
    @classmethod
    def options(cls, wrapped, _disambiguate_from_model_properties: bool = ...): ...
    @classmethod
    def slots(cls): ...
    def __init__(self, config: Incomplete | None = ..., **kwargs) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def copy(self, **kwargs): ...
    def items(self) -> None: ...

class ReadOptions(Options):
    __slots__ = ("read_consistency", "read_policy", "transaction")
    def __init__(self, config: Incomplete | None = ..., **kwargs) -> None: ...



from collections.abc import Sequence
from typing import Any, Final

CLARA_AGX_XAVIER: Final = "CLARA_AGX_XAVIER"
JETSON_NX: Final = "JETSON_NX"
JETSON_XAVIER: Final = "JETSON_XAVIER"
JETSON_TX2: Final = "JETSON_TX2"
JETSON_TX1: Final = "JETSON_TX1"
JETSON_NANO: Final = "JETSON_NANO"
JETSON_TX2_NX: Final = "JETSON_TX2_NX"
JETSON_ORIN: Final = "JETSON_ORIN"
JETSON_ORIN_NX: Final = "JETSON_ORIN_NX"
JETSON_ORIN_NANO: Final = "JETSON_ORIN_NANO"
JETSON_THOR_REFERENCE: Final = "JETSON_THOR_REFERENCE"

JETSON_MODELS: list[str]

JETSON_ORIN_NX_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None, int]]
compats_jetson_orins_nx: Sequence[str]
compats_jetson_orins_nano: Sequence[str]

JETSON_ORIN_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None, int]]
compats_jetson_orins: Sequence[str]

CLARA_AGX_XAVIER_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_clara_agx_xavier: Sequence[str]

JETSON_NX_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_nx: Sequence[str]

JETSON_XAVIER_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_xavier: Sequence[str]

JETSON_TX2_NX_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_tx2_nx: Sequence[str]

JETSON_TX2_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_tx2: Sequence[str]

JETSON_TX1_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_tx1: Sequence[str]

JETSON_NANO_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_nano: Sequence[str]

JETSON_THOR_REFERENCE_PIN_DEFS: list[tuple[int, str, str, int, int, str, str, str | None, int | None]]
compats_jetson_thor_reference: Sequence[str]

jetson_gpio_data: dict[str, tuple[list[tuple[int, str, str, int, int, str, str, str | None, int | None]], dict[str, Any]]]

class ChannelInfo:
    channel: int
    chip_fd: int | None
    line_handle: int | None
    line_offset: int
    direction: int | None
    edge: int | None
    consumer: str
    gpio_name: str
    gpio_chip: str
    pwm_chip_dir: str
    pwm_id: int
    reg_addr: int | None
    def __init__(
        self,
        channel: int,
        line_offset: int,
        gpio_name: str,
        gpio_chip: str,
        pwm_chip_dir: str,
        pwm_id: int,
        reg_addr: int | None = None,
    ) -> None: ...

ids_warned: bool

def find_pmgr_board(prefix: str) -> str | None: ...
def warn_if_not_carrier_board(*carrier_boards: str) -> None: ...
def get_compatibles(compatible_path: str) -> list[str]: ...
def get_model() -> str: ...
def get_data() -> tuple[str, Any, dict[str, dict[Any, ChannelInfo]]]: ...



from typing import TypedDict, type_check_only

@type_check_only
class _GraphqlVariables(TypedDict, total=False):
    countryCode: str
    extendedAddress: str
    locality: str
    postalCode: str
    region: str
    streetAddress: str

class ShippingAddressInput:
    def __init__(
        self,
        country_code_alpha2: str | None = None,
        extended_address: str | None = None,
        locality: str | None = None,
        postal_code: str | None = None,
        region: str | None = None,
        street_address: str | None = None,
    ) -> None: ...
    def to_graphql_variables(self) -> _GraphqlVariables: ...



from _typeshed import StrPath
from collections.abc import Callable, Iterable
from shlex import _ShlexInstream
from typing import ClassVar, Final, Literal, NoReturn
from typing_extensions import Never, deprecated

from ...version import LooseVersion
from . import unix

def get_msvcr() -> list[str]: ...

class Compiler(unix.Compiler):
    compiler_type: ClassVar[str]
    obj_extension: ClassVar[str]
    static_lib_extension: ClassVar[str]
    shared_lib_extension: ClassVar[str]
    dylib_lib_extension: ClassVar[str]
    static_lib_format: ClassVar[str]
    shared_lib_format: ClassVar[str]
    dylib_lib_format: ClassVar[str]
    exe_extension: ClassVar[str]
    cc: str
    cxx: str
    linker_dll: str
    linker_dll_cxx: str
    dll_libraries: list[str]
    def __init__(self, verbose: bool = False, force: bool = False) -> None: ...
    @property
    @deprecated(
        "gcc_version attribute of CygwinCCompiler is deprecated. "
        "Instead of returning actual gcc version a fixed value 11.2.0 is returned."
    )
    def gcc_version(self) -> LooseVersion: ...
    # `objects` and `libraries` uses list methods
    def link(
        self,
        target_desc: str,
        objects: list[str],  # type: ignore[override]
        output_filename: str,
        output_dir: str | None = None,
        libraries: list[str] | None = None,  # type: ignore[override]
        library_dirs: list[str] | tuple[str, ...] | None = None,
        runtime_library_dirs: list[str] | tuple[str, ...] | None = None,
        export_symbols: Iterable[str] | None = None,
        debug: bool = False,
        extra_preargs: list[str] | None = None,
        extra_postargs: list[str] | None = None,
        build_temp: StrPath | None = None,
        target_lang: str | None = None,
    ) -> None: ...
    # cygwin doesn't support rpath; prints a warning and returns an empty list
    def runtime_library_dir_option(self, dir: str) -> list[Never]: ...  # type: ignore[override]
    @property
    def out_extensions(self) -> dict[str, str]: ...

class MinGW32Compiler(Compiler):
    compiler_type: ClassVar[str]
    def __init__(self, verbose: bool = False, force: bool = False) -> None: ...
    def runtime_library_dir_option(self, dir: str) -> NoReturn: ...

CONFIG_H_OK: Final = "ok"
CONFIG_H_NOTOK: Final = "not ok"
CONFIG_H_UNCERTAIN: Final = "uncertain"

def check_config_h() -> tuple[Literal["ok", "not ok", "uncertain"], str]: ...
def is_cygwincc(cc: str | _ShlexInstream) -> bool: ...

get_versions: Callable[[], tuple[LooseVersion | None, ...]] | None



from _typeshed import StrPath
from py_compile import PycInvalidationMode
from typing import Any, Protocol, type_check_only

__all__ = ["compile_dir", "compile_file", "compile_path"]

@type_check_only
class _SupportsSearch(Protocol):
    def search(self, string: str, /) -> Any: ...

def compile_dir(
    dir: StrPath,
    maxlevels: int | None = None,
    ddir: StrPath | None = None,
    force: bool = False,
    rx: _SupportsSearch | None = None,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    workers: int = 1,
    invalidation_mode: PycInvalidationMode | None = None,
    *,
    stripdir: StrPath | None = None,
    prependdir: StrPath | None = None,
    limit_sl_dest: StrPath | None = None,
    hardlink_dupes: bool = False,
) -> bool: ...
def compile_file(
    fullname: StrPath,
    ddir: StrPath | None = None,
    force: bool = False,
    rx: _SupportsSearch | None = None,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    invalidation_mode: PycInvalidationMode | None = None,
    *,
    stripdir: StrPath | None = None,
    prependdir: StrPath | None = None,
    limit_sl_dest: StrPath | None = None,
    hardlink_dupes: bool = False,
) -> bool: ...
def compile_path(
    skip_curdir: bool = ...,
    maxlevels: int = 0,
    force: bool = False,
    quiet: int = 0,
    legacy: bool = False,
    optimize: int = -1,
    invalidation_mode: PycInvalidationMode | None = None,
) -> bool: ...



import sys
from _typeshed import StrOrBytesPath
from collections.abc import Callable, Iterator, MutableMapping
from dbm import _TFlags
from types import TracebackType
from typing import Any, TypeVar, overload
from typing_extensions import Self

__all__ = ["Shelf", "BsdDbShelf", "DbfilenameShelf", "open"]
if sys.version_info >= (3, 15):
    __all__ += ["ShelveError"]

_T = TypeVar("_T")
_VT = TypeVar("_VT")

if sys.version_info >= (3, 15):
    class ShelveError(Exception): ...

class Shelf(MutableMapping[str, _VT]):
    if sys.version_info >= (3, 15):
        def __init__(
            self,
            dict: MutableMapping[bytes, bytes],
            protocol: int | None = None,
            writeback: bool = False,
            keyencoding: str = "utf-8",
            *,
            serializer: Callable[[Any], bytes] | None = None,
            deserializer: Callable[[bytes], Any] | None = None,
        ) -> None: ...

    else:
        def __init__(
            self,
            dict: MutableMapping[bytes, bytes],
            protocol: int | None = None,
            writeback: bool = False,
            keyencoding: str = "utf-8",
        ) -> None: ...

    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...

    @overload  # type: ignore[override]
    def get(self, key: str, default: None = None) -> _VT | None: ...
    @overload
    def get(self, key: str, default: _VT) -> _VT: ...
    @overload
    def get(self, key: str, default: _T) -> _VT | _T: ...

    def __getitem__(self, key: str) -> _VT: ...
    def __setitem__(self, key: str, value: _VT) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __contains__(self, key: str) -> bool: ...  # type: ignore[override]
    def __enter__(self) -> Self: ...
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None
    ) -> None: ...
    def __del__(self) -> None: ...
    def close(self) -> None: ...
    def sync(self) -> None: ...
    if sys.version_info >= (3, 15):
        def reorganize(self) -> None: ...

class BsdDbShelf(Shelf[_VT]):
    def set_location(self, key: str) -> tuple[str, _VT]: ...
    def next(self) -> tuple[str, _VT]: ...
    def previous(self) -> tuple[str, _VT]: ...
    def first(self) -> tuple[str, _VT]: ...
    def last(self) -> tuple[str, _VT]: ...

class DbfilenameShelf(Shelf[_VT]):
    if sys.version_info >= (3, 15):
        def __init__(
            self,
            filename: StrOrBytesPath,
            flag: _TFlags = "c",
            protocol: int | None = None,
            writeback: bool = False,
            *,
            serializer: Callable[[Any], bytes] | None = None,
            deserializer: Callable[[bytes], Any] | None = None,
        ) -> None: ...

    elif sys.version_info >= (3, 11):
        def __init__(
            self, filename: StrOrBytesPath, flag: _TFlags = "c", protocol: int | None = None, writeback: bool = False
        ) -> None: ...

    else:
        def __init__(self, filename: str, flag: _TFlags = "c", protocol: int | None = None, writeback: bool = False) -> None: ...

if sys.version_info >= (3, 15):
    def open(
        filename: StrOrBytesPath,
        flag: _TFlags = "c",
        protocol: int | None = None,
        writeback: bool = False,
        *,
        serializer: Callable[[Any], bytes] | None = None,
        deserializer: Callable[[bytes], Any] | None = None,
    ) -> Shelf[Any]: ...

elif sys.version_info >= (3, 11):
    def open(
        filename: StrOrBytesPath, flag: _TFlags = "c", protocol: int | None = None, writeback: bool = False
    ) -> Shelf[Any]: ...

else:
    def open(filename: str, flag: _TFlags = "c", protocol: int | None = None, writeback: bool = False) -> Shelf[Any]: ...



from _typeshed import StrOrBytesPath
from collections.abc import Sequence

from _remote_debugging import AwaitedInfo, InterpreterInfo

from .collector import _Frame, _Timestamps
from .stack_collector import StackTraceCollector

class JsonlCollector(StackTraceCollector):
    def __init__(self, sample_interval_usec: int, *, skip_idle: bool = False, mode: int | None = None) -> None: ...
    def collect(
        self, stack_frames: Sequence[InterpreterInfo] | Sequence[AwaitedInfo], timestamps_us: _Timestamps = None
    ) -> None: ...
    def export(self, filename: StrOrBytesPath) -> None: ...
    def process_frames(self, frames: Sequence[_Frame], _thread_id: int, weight: int = 1) -> None: ...



from _typeshed import Incomplete, SupportsWrite
from collections.abc import Iterable
from typing import TypeVar

from pygments.formatter import Formatter
from pygments.token import _TokenType

_T = TypeVar("_T", str, bytes)

__all__ = ["SvgFormatter"]

class SvgFormatter(Formatter[_T]):
    nowrap: Incomplete
    fontfamily: Incomplete
    fontsize: Incomplete
    xoffset: Incomplete
    yoffset: Incomplete
    ystep: Incomplete
    spacehack: Incomplete
    linenos: Incomplete
    linenostart: Incomplete
    linenostep: Incomplete
    linenowidth: Incomplete
    def format_unencoded(self, tokensource: Iterable[tuple[_TokenType, str]], outfile: SupportsWrite[str]) -> None: ...



from _typeshed import Incomplete
from collections.abc import Generator
from contextlib import contextmanager
from typing import NoReturn

from authlib.oauth2 import ResourceProtector as _ResourceProtector

class ResourceProtector(_ResourceProtector):
    def raise_error_response(self, error) -> NoReturn: ...
    def acquire_token(self, scopes=None, **kwargs): ...
    @contextmanager
    def acquire(self, scopes=None) -> Generator[Incomplete]: ...
    def __call__(self, scopes=None, optional=False, **kwargs): ...

current_token: Incomplete



from collections.abc import Iterable
from re import Pattern

from .ast import Node

def parse_nth(input: str | Iterable[Node]) -> tuple[int, int] | None: ...
def parse_b(tokens: Iterable[Node], a: int) -> tuple[int, int] | None: ...
def parse_signless_b(tokens: Iterable[Node], a: int, b_sign: int) -> tuple[int, int] | None: ...
def parse_end(tokens: Iterable[Node], a: int, b: int) -> tuple[int, int] | None: ...

N_DASH_DIGITS_RE: Pattern[str]



from _typeshed import Incomplete

from authlib.oauth2.rfc6749 import AuthorizationCodeMixin, TokenMixin

class OAuth2AuthorizationCodeMixin(AuthorizationCodeMixin):
    code: Incomplete
    client_id: Incomplete
    redirect_uri: Incomplete
    response_type: Incomplete
    scope: Incomplete
    nonce: Incomplete
    auth_time: Incomplete
    acr: Incomplete
    amr: Incomplete
    code_challenge: Incomplete
    code_challenge_method: Incomplete
    def is_expired(self) -> bool: ...
    def get_redirect_uri(self): ...
    def get_scope(self): ...
    def get_auth_time(self): ...
    def get_acr(self): ...
    def get_amr(self): ...
    def get_nonce(self): ...

class OAuth2TokenMixin(TokenMixin):
    client_id: Incomplete
    token_type: Incomplete
    access_token: Incomplete
    refresh_token: Incomplete
    scope: Incomplete
    issued_at: Incomplete
    access_token_revoked_at: Incomplete
    refresh_token_revoked_at: Incomplete
    expires_in: Incomplete
    def check_client(self, client) -> bool: ...
    def get_scope(self): ...
    def get_expires_in(self): ...
    def is_revoked(self): ...
    def is_expired(self) -> bool: ...



import datetime
import enum
import sys
from _typeshed import Unused
from collections.abc import Iterable
from time import struct_time
from typing import ClassVar, Final, TypeAlias, overload

__all__ = [
    "FRIDAY",
    "MONDAY",
    "SATURDAY",
    "SUNDAY",
    "THURSDAY",
    "TUESDAY",
    "WEDNESDAY",
    "IllegalMonthError",
    "IllegalWeekdayError",
    "setfirstweekday",
    "firstweekday",
    "isleap",
    "leapdays",
    "weekday",
    "monthrange",
    "monthcalendar",
    "prmonth",
    "month",
    "prcal",
    "calendar",
    "timegm",
    "month_name",
    "month_abbr",
    "day_name",
    "day_abbr",
    "Calendar",
    "TextCalendar",
    "HTMLCalendar",
    "LocaleTextCalendar",
    "LocaleHTMLCalendar",
    "weekheader",
]

if sys.version_info >= (3, 12):
    __all__ += [
        "Day",
        "Month",
        "JANUARY",
        "FEBRUARY",
        "MARCH",
        "APRIL",
        "MAY",
        "JUNE",
        "JULY",
        "AUGUST",
        "SEPTEMBER",
        "OCTOBER",
        "NOVEMBER",
        "DECEMBER",
    ]
if sys.version_info >= (3, 15):
    __all__ += ["standalone_month_name", "standalone_month_abbr"]

_LocaleType: TypeAlias = tuple[str | None, str | None]

class IllegalMonthError(ValueError, IndexError):
    month: int
    def __init__(self, month: int) -> None: ...

class IllegalWeekdayError(ValueError):
    weekday: int
    def __init__(self, weekday: int) -> None: ...

def isleap(year: int) -> bool: ...
def leapdays(y1: int, y2: int) -> int: ...
def weekday(year: int, month: int, day: int) -> int: ...
def monthrange(year: int, month: int) -> tuple[int, int]: ...

class Calendar:
    firstweekday: int
    def __init__(self, firstweekday: int = 0) -> None: ...
    def getfirstweekday(self) -> int: ...
    def setfirstweekday(self, firstweekday: int) -> None: ...
    def iterweekdays(self) -> Iterable[int]: ...
    def itermonthdates(self, year: int, month: int) -> Iterable[datetime.date]: ...
    def itermonthdays2(self, year: int, month: int) -> Iterable[tuple[int, int]]: ...
    def itermonthdays(self, year: int, month: int) -> Iterable[int]: ...
    def monthdatescalendar(self, year: int, month: int) -> list[list[datetime.date]]: ...
    def monthdays2calendar(self, year: int, month: int) -> list[list[tuple[int, int]]]: ...
    def monthdayscalendar(self, year: int, month: int) -> list[list[int]]: ...
    def yeardatescalendar(self, year: int, width: int = 3) -> list[list[list[list[datetime.date]]]]: ...
    def yeardays2calendar(self, year: int, width: int = 3) -> list[list[list[list[tuple[int, int]]]]]: ...
    def yeardayscalendar(self, year: int, width: int = 3) -> list[list[list[list[int]]]]: ...
    def itermonthdays3(self, year: int, month: int) -> Iterable[tuple[int, int, int]]: ...
    def itermonthdays4(self, year: int, month: int) -> Iterable[tuple[int, int, int, int]]: ...

class TextCalendar(Calendar):
    def prweek(self, theweek: Iterable[tuple[int, int]], width: int) -> None: ...
    def formatday(self, day: int, weekday: int, width: int) -> str: ...
    def formatweek(self, theweek: Iterable[tuple[int, int]], width: int) -> str: ...
    def formatweekday(self, day: int, width: int) -> str: ...
    def formatweekheader(self, width: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, width: int, withyear: bool = True) -> str: ...
    def prmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> None: ...
    def formatmonth(self, theyear: int, themonth: int, w: int = 0, l: int = 0) -> str: ...
    def formatyear(self, theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...
    def pryear(self, theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None: ...

def firstweekday() -> int: ...
def monthcalendar(year: int, month: int) -> list[list[int]]: ...
def prweek(theweek: int, width: int) -> None: ...
def week(theweek: int, width: int) -> str: ...
def weekheader(width: int) -> str: ...
def prmonth(theyear: int, themonth: int, w: int = 0, l: int = 0) -> None: ...
def month(theyear: int, themonth: int, w: int = 0, l: int = 0) -> str: ...
def calendar(theyear: int, w: int = 2, l: int = 1, c: int = 6, m: int = 3) -> str: ...
def prcal(theyear: int, w: int = 0, l: int = 0, c: int = 6, m: int = 3) -> None: ...

class HTMLCalendar(Calendar):
    cssclasses: ClassVar[list[str]]
    cssclass_noday: ClassVar[str]
    cssclasses_weekday_head: ClassVar[list[str]]
    cssclass_month_head: ClassVar[str]
    cssclass_month: ClassVar[str]
    cssclass_year: ClassVar[str]
    cssclass_year_head: ClassVar[str]
    def formatday(self, day: int, weekday: int) -> str: ...
    def formatweek(self, theweek: int) -> str: ...
    def formatweekday(self, day: int) -> str: ...
    def formatweekheader(self) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    def formatmonth(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...
    if sys.version_info >= (3, 15):
        def formatmonthpage(
            self, theyear: int, themonth: int, width: int = 3, css: str | None = "calendar.css", encoding: str | None = None
        ) -> bytes: ...

    def formatyear(self, theyear: int, width: int = 3) -> str: ...
    def formatyearpage(
        self, theyear: int, width: int = 3, css: str | None = "calendar.css", encoding: str | None = None
    ) -> bytes: ...

class different_locale:
    def __init__(self, locale: _LocaleType) -> None: ...
    def __enter__(self) -> None: ...
    def __exit__(self, *args: Unused) -> None: ...

class LocaleTextCalendar(TextCalendar):
    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...

class LocaleHTMLCalendar(HTMLCalendar):
    def __init__(self, firstweekday: int = 0, locale: _LocaleType | None = None) -> None: ...
    def formatweekday(self, day: int) -> str: ...
    def formatmonthname(self, theyear: int, themonth: int, withyear: bool = True) -> str: ...

c: TextCalendar

def setfirstweekday(firstweekday: int) -> None: ...
def format(cols: int, colwidth: int = 20, spacing: int = 6) -> str: ...
def formatstring(cols: Iterable[str], colwidth: int = 20, spacing: int = 6) -> str: ...
def timegm(tuple: tuple[int, ...] | struct_time) -> int: ...

# Data attributes
class _localized_month:
    format: str
    def __init__(self, format: str) -> None: ...

    @overload
    def __getitem__(self, i: int) -> str: ...
    @overload
    def __getitem__(self, i: slice) -> list[str]: ...

    def __len__(self) -> int: ...

class _localized_day:
    format: str
    def __init__(self, format: str) -> None: ...

    @overload
    def __getitem__(self, i: int) -> str: ...
    @overload
    def __getitem__(self, i: slice) -> list[str]: ...

    def __len__(self) -> int: ...

day_name: _localized_day
day_abbr: _localized_day
month_name: _localized_month
month_abbr: _localized_month

if sys.version_info >= (3, 12):
    class Month(enum.IntEnum):
        JANUARY = 1
        FEBRUARY = 2
        MARCH = 3
        APRIL = 4
        MAY = 5
        JUNE = 6
        JULY = 7
        AUGUST = 8
        SEPTEMBER = 9
        OCTOBER = 10
        NOVEMBER = 11
        DECEMBER = 12

    JANUARY: Final = Month.JANUARY
    FEBRUARY: Final = Month.FEBRUARY
    MARCH: Final = Month.MARCH
    APRIL: Final = Month.APRIL
    MAY: Final = Month.MAY
    JUNE: Final = Month.JUNE
    JULY: Final = Month.JULY
    AUGUST: Final = Month.AUGUST
    SEPTEMBER: Final = Month.SEPTEMBER
    OCTOBER: Final = Month.OCTOBER
    NOVEMBER: Final = Month.NOVEMBER
    DECEMBER: Final = Month.DECEMBER

    class Day(enum.IntEnum):
        MONDAY = 0
        TUESDAY = 1
        WEDNESDAY = 2
        THURSDAY = 3
        FRIDAY = 4
        SATURDAY = 5
        SUNDAY = 6

    MONDAY: Final = Day.MONDAY
    TUESDAY: Final = Day.TUESDAY
    WEDNESDAY: Final = Day.WEDNESDAY
    THURSDAY: Final = Day.THURSDAY
    FRIDAY: Final = Day.FRIDAY
    SATURDAY: Final = Day.SATURDAY
    SUNDAY: Final = Day.SUNDAY
else:
    MONDAY: Final = 0
    TUESDAY: Final = 1
    WEDNESDAY: Final = 2
    THURSDAY: Final = 3
    FRIDAY: Final = 4
    SATURDAY: Final = 5
    SUNDAY: Final = 6

EPOCH: Final = 1970

if sys.version_info >= (3, 15):
    standalone_month_name: _localized_month
    standalone_month_abbr: _localized_month



from typing import ClassVar

from gunicorn.http.parser import Parser
from gunicorn.uwsgi.message import UWSGIRequest

class UWSGIParser(Parser):
    mesg_class: ClassVar[type[UWSGIRequest]]  # type: ignore[assignment]



from collections.abc import Iterable
from typing import Any

class EndpointConfig(dict[str, Any]):
    def __init__(
        self,
        version: str,
        aliases: list[str] | None = None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        ipv4_address: str | None = None,
        ipv6_address: str | None = None,
        link_local_ips: list[str] | None = None,
        driver_opt: dict[str, str] | None = None,
        mac_address: str | None = None,
    ) -> None: ...

class NetworkingConfig(dict[str, Any]):
    def __init__(self, endpoints_config: EndpointConfig | None = None) -> None: ...

class IPAMConfig(dict[str, Any]):
    def __init__(
        self, driver: str = "default", pool_configs: list[IPAMPool] | None = None, options: dict[str, str] | None = None
    ) -> None: ...

class IPAMPool(dict[str, Any]):
    def __init__(
        self,
        subnet: str | None = None,
        iprange: str | None = None,
        gateway: str | None = None,
        aux_addresses: dict[str, str] | None = None,
    ) -> None: ...



from _typeshed import Incomplete

from braintree.error_result import ErrorResult
from braintree.graphql import CreateLocalPaymentContextInput
from braintree.successful_result import SuccessfulResult

class LocalPaymentContextGateway:
    CREATE_LOCAL_PAYMENT_CONTEXT: str
    FIND_LOCAL_PAYMENT_CONTEXT: str
    gateway: Incomplete
    graphql_client: Incomplete
    def __init__(self, gateway) -> None: ...
    def create(self, input: CreateLocalPaymentContextInput) -> SuccessfulResult | ErrorResult: ...
    def find(self, id) -> SuccessfulResult | ErrorResult: ...



from cProfile import Profile as Profile, run as run, runctx as runctx
from types import CodeType
from typing import TypeAlias

__all__ = ("run", "runctx", "Profile")

_Label: TypeAlias = tuple[str, int, str]

def label(code: str | CodeType) -> _Label: ...  # undocumented



from _typeshed import Incomplete, ReadableBuffer
from collections.abc import Iterable
from typing import SupportsBytes, SupportsIndex

from .models import JWSAlgorithm, JWSObject

class JsonWebSignature:
    REGISTERED_HEADER_PARAMETER_NAMES: frozenset[str]
    MAX_CONTENT_LENGTH: int
    ALGORITHMS_REGISTRY: dict[str, JWSAlgorithm]
    def __init__(self, algorithms=None, private_headers=None) -> None: ...
    @classmethod
    def register_algorithm(cls, algorithm: JWSAlgorithm) -> None: ...
    def serialize_compact(self, protected, payload, key) -> bytes: ...
    def deserialize_compact(
        self, s: str | bytes | float | Iterable[SupportsIndex] | SupportsIndex | SupportsBytes | ReadableBuffer, key, decode=None
    ) -> JWSObject: ...
    def serialize_json(self, header_obj, payload, key) -> dict[str, Incomplete]: ...
    def deserialize_json(self, obj, key, decode=None) -> JWSObject: ...
    def serialize(self, header, payload, key) -> dict[str, Incomplete] | bytes: ...
    def deserialize(self, s, key, decode=None) -> JWSObject: ...



from typing import TypedDict, type_check_only
from typing_extensions import Self

from braintree.graphql.inputs.phone_input import PhoneInput, _GraphqlVariables as _PhoneInputGraphqlVariables

@type_check_only
class _GraphqlVariables(TypedDict, total=False):
    email: str
    hashedEmail: str
    phone: _PhoneInputGraphqlVariables
    hashedPhoneNumber: str
    deviceFingerprintId: str
    paypalAppInstalled: bool
    venmoAppInstalled: bool
    userAgent: str

class CustomerSessionInput:
    def __init__(
        self,
        email: str | None = None,
        hashed_email: str | None = None,
        phone: PhoneInput | None = None,
        hashed_phone_number: str | None = None,
        device_fingerprint_id: str | None = None,
        paypal_app_installed: bool | None = None,
        venmo_app_installed: bool | None = None,
        user_agent: str | None = None,
    ) -> None: ...
    def to_graphql_variables(self) -> _GraphqlVariables: ...
    @staticmethod
    def builder() -> Builder: ...

    class Builder:
        def __init__(self) -> None: ...
        def email(self, email: str) -> Self: ...
        def hashed_email(self, hashed_email: str) -> Self: ...
        def phone(self, phone: PhoneInput) -> Self: ...
        def hashed_phone_number(self, hashed_phone_number: str) -> Self: ...
        def device_fingerprint_id(self, device_fingerprint_id: str) -> Self: ...
        def paypal_app_installed(self, paypal_app_installed: bool) -> Self: ...
        def venmo_app_installed(self, venmo_app_installed: bool) -> Self: ...
        def user_agent(self, user_agent: str) -> Self: ...
        def build(self) -> CustomerSessionInput: ...



from collections.abc import Generator
from re import Pattern
from typing import Final

RE_DATA_URI: Final[Pattern[str]]  # undocumented

class DataURIError(ValueError): ...

class ParsedDataURI:
    media_type: str | None
    data: bytes
    uri: str

    def __init__(self, media_type: str | None, data: bytes, uri: str) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...

def parse(uri: str) -> ParsedDataURI: ...
def discover(s: str) -> Generator[ParsedDataURI]: ...



from typing import Literal

class Node:
    source_line: int
    source_column: int
    type: str

    def __init__(self, source_line: int, source_column: int) -> None: ...
    def serialize(self) -> str: ...

class ParseError(Node):
    type: Literal["error"]
    kind: str
    message: str
    repr_format: str
    def __init__(self, line: int, column: int, kind: str, message: str) -> None: ...

class Comment(Node):
    type: Literal["comment"]
    value: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str) -> None: ...

class WhitespaceToken(Node):
    type: Literal["whitespace"]
    value: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str) -> None: ...

class LiteralToken(Node):
    type: Literal["literal"]
    value: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...

class IdentToken(Node):
    type: Literal["ident"]
    value: str
    lower_value: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str) -> None: ...

class AtKeywordToken(Node):
    type: Literal["at-keyword"]
    value: str
    lower_value: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str) -> None: ...

class HashToken(Node):
    type: Literal["hash"]
    value: str
    is_identifier: bool
    repr_format: str
    def __init__(self, line: int, column: int, value: str, is_identifier: bool) -> None: ...

class StringToken(Node):
    type: Literal["string"]
    value: str
    representation: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str, representation: str) -> None: ...

class URLToken(Node):
    type: Literal["url"]
    value: str
    representation: str
    repr_format: str
    def __init__(self, line: int, column: int, value: str, representation: str) -> None: ...

class UnicodeRangeToken(Node):
    type: Literal["unicode-range"]
    start: int
    end: int
    repr_format: str
    def __init__(self, line: int, column: int, start: int, end: int) -> None: ...

class NumberToken(Node):
    type: Literal["number"]
    value: float
    int_value: int | None
    is_integer: bool
    representation: str
    repr_format: str
    def __init__(self, line: int, column: int, value: float, int_value: int | None, representation: str) -> None: ...

class PercentageToken(Node):
    type: Literal["percentage"]
    value: float
    int_value: int | None
    is_integer: bool
    representation: str
    repr_format: str
    def __init__(self, line: int, column: int, value: float, int_value: int | None, representation: str) -> None: ...

class DimensionToken(Node):
    type: Literal["dimension"]
    value: float
    int_value: int | None
    is_integer: bool
    representation: str
    unit: str
    lower_unit: str
    repr_format: str
    def __init__(self, line: int, column: int, value: float, int_value: int | None, representation: str, unit: str) -> None: ...

class ParenthesesBlock(Node):
    type: Literal["() block"]
    content: list[Node]
    repr_format: str
    def __init__(self, line: int, column: int, content: list[Node]) -> None: ...

class SquareBracketsBlock(Node):
    type: Literal["[] block"]
    content: list[Node]
    repr_format: str
    def __init__(self, line: int, column: int, content: list[Node]) -> None: ...

class CurlyBracketsBlock(Node):
    type: Literal["{} block"]
    content: list[Node]
    repr_format: str
    def __init__(self, line: int, column: int, content: list[Node]) -> None: ...

class FunctionBlock(Node):
    type: Literal["function"]
    name: str
    lower_name: str
    arguments: list[Node]
    repr_format: str
    def __init__(self, line: int, column: int, name: str, arguments: list[Node]) -> None: ...

class Declaration(Node):
    type: Literal["declaration"]
    name: str
    lower_name: str
    value: list[Node]
    important: bool
    repr_format: str
    def __init__(self, line: int, column: int, name: str, lower_name: str, value: list[Node], important: bool) -> None: ...

class QualifiedRule(Node):
    type: Literal["qualified-rule"]
    prelude: list[Node]
    content: list[Node]
    repr_format: str
    def __init__(self, line: int, column: int, prelude: list[Node], content: list[Node]) -> None: ...

class AtRule(Node):
    type: Literal["at-rule"]
    at_keyword: str
    lower_at_keyword: str
    prelude: list[Node]
    content: list[Node] | None
    repr_format: str
    def __init__(
        self, line: int, column: int, at_keyword: str, lower_at_keyword: str, prelude: list[Node], content: list[Node] | None
    ) -> None: ...



from typing import Final

STGC_DEFAULT: Final = 0
STGC_OVERWRITE: Final = 1
STGC_ONLYIFCURRENT: Final = 2
STGC_DANGEROUSLYCOMMITMERELYTODISKCACHE: Final = 4
STGC_CONSOLIDATE: Final = 8
STGTY_STORAGE: Final = 1
STGTY_STREAM: Final = 2
STGTY_LOCKBYTES: Final = 3
STGTY_PROPERTY: Final = 4
STREAM_SEEK_SET: Final = 0
STREAM_SEEK_CUR: Final = 1
STREAM_SEEK_END: Final = 2
LOCK_WRITE: Final = 1
LOCK_EXCLUSIVE: Final = 2
LOCK_ONLYONCE: Final = 4
CWCSTORAGENAME: Final = 32
STGM_DIRECT: Final = 0x00000000
STGM_TRANSACTED: Final = 0x00010000
STGM_SIMPLE: Final = 0x08000000
STGM_READ: Final = 0x00000000
STGM_WRITE: Final = 0x00000001
STGM_READWRITE: Final = 0x00000002
STGM_SHARE_DENY_NONE: Final = 0x00000040
STGM_SHARE_DENY_READ: Final = 0x00000030
STGM_SHARE_DENY_WRITE: Final = 0x00000020
STGM_SHARE_EXCLUSIVE: Final = 0x00000010
STGM_PRIORITY: Final = 0x00040000
STGM_DELETEONRELEASE: Final = 0x04000000
STGM_NOSCRATCH: Final = 0x00100000
STGM_CREATE: Final = 0x00001000
STGM_CONVERT: Final = 0x00020000
STGM_FAILIFTHERE: Final = 0x00000000
STGM_NOSNAPSHOT: Final = 0x00200000
ASYNC_MODE_COMPATIBILITY: Final = 0x00000001
ASYNC_MODE_DEFAULT: Final = 0x00000000
STGTY_REPEAT: Final = 0x00000100
STG_TOEND: Final = 0xFFFFFFFF
STG_LAYOUT_SEQUENTIAL: Final = 0x00000000
STG_LAYOUT_INTERLEAVED: Final = 0x00000001
COM_RIGHTS_EXECUTE: Final = 1
COM_RIGHTS_EXECUTE_LOCAL: Final = 2
COM_RIGHTS_EXECUTE_REMOTE: Final = 4
COM_RIGHTS_ACTIVATE_LOCAL: Final = 8
COM_RIGHTS_ACTIVATE_REMOTE: Final = 16
STGFMT_DOCUMENT: Final = 0
STGFMT_STORAGE: Final = 0
STGFMT_NATIVE: Final = 1
STGFMT_FILE: Final = 3
STGFMT_ANY: Final = 4
STGFMT_DOCFILE: Final = 5
PID_DICTIONARY: Final = 0
PID_CODEPAGE: Final = 1
PID_FIRST_USABLE: Final = 2
PID_FIRST_NAME_DEFAULT: Final = 4095
PID_LOCALE: Final = -2147483648
PID_MODIFY_TIME: Final = -2147483647
PID_SECURITY: Final = -2147483646
PID_BEHAVIOR: Final = -2147483645
PID_ILLEGAL: Final = -1
PID_MIN_READONLY: Final = -2147483648
PID_MAX_READONLY: Final = -1073741825
PIDDI_THUMBNAIL: Final = 0x00000002
PIDSI_TITLE: Final = 2
PIDSI_SUBJECT: Final = 3
PIDSI_AUTHOR: Final = 4
PIDSI_KEYWORDS: Final = 5
PIDSI_COMMENTS: Final = 6
PIDSI_TEMPLATE: Final = 7
PIDSI_LASTAUTHOR: Final = 8
PIDSI_REVNUMBER: Final = 9
PIDSI_EDITTIME: Final = 10
PIDSI_LASTPRINTED: Final = 11
PIDSI_CREATE_DTM: Final = 12
PIDSI_LASTSAVE_DTM: Final = 13
PIDSI_PAGECOUNT: Final = 14
PIDSI_WORDCOUNT: Final = 15
PIDSI_CHARCOUNT: Final = 16
PIDSI_THUMBNAIL: Final = 17
PIDSI_APPNAME: Final = 18
PIDSI_DOC_SECURITY: Final = 19
PIDDSI_CATEGORY: Final = 2
PIDDSI_PRESFORMAT: Final = 3
PIDDSI_BYTECOUNT: Final = 4
PIDDSI_LINECOUNT: Final = 5
PIDDSI_PARCOUNT: Final = 6
PIDDSI_SLIDECOUNT: Final = 7
PIDDSI_NOTECOUNT: Final = 8
PIDDSI_HIDDENCOUNT: Final = 9
PIDDSI_MMCLIPCOUNT: Final = 10
PIDDSI_SCALE: Final = 11
PIDDSI_HEADINGPAIR: Final = 12
PIDDSI_DOCPARTS: Final = 13
PIDDSI_MANAGER: Final = 14
PIDDSI_COMPANY: Final = 15
PIDDSI_LINKSDIRTY: Final = 16
PIDMSI_EDITOR: Final = 2
PIDMSI_SUPPLIER: Final = 3
PIDMSI_SOURCE: Final = 4
PIDMSI_SEQUENCE_NO: Final = 5
PIDMSI_PROJECT: Final = 6
PIDMSI_STATUS: Final = 7
PIDMSI_OWNER: Final = 8
PIDMSI_RATING: Final = 9
PIDMSI_PRODUCTION: Final = 10
PIDMSI_COPYRIGHT: Final = 11
PROPSETFLAG_DEFAULT: Final = 0
PROPSETFLAG_NONSIMPLE: Final = 1
PROPSETFLAG_ANSI: Final = 2
PROPSETFLAG_UNBUFFERED: Final = 4
PROPSETFLAG_CASE_SENSITIVE: Final = 8
STGMOVE_MOVE: Final = 0
STGMOVE_COPY: Final = 1
STGMOVE_SHALLOWCOPY: Final = 2



from typing import Final, Literal, TypeAlias

_Cats: TypeAlias = Literal[
    "Cc",
    "Cf",
    "Cn",
    "Co",
    "Cs",
    "Ll",
    "Lm",
    "Lo",
    "Lt",
    "Lu",
    "Mc",
    "Me",
    "Mn",
    "Nd",
    "Nl",
    "No",
    "Pc",
    "Pd",
    "Pe",
    "Pf",
    "Pi",
    "Po",
    "Ps",
    "Sc",
    "Sk",
    "Sm",
    "So",
    "Zl",
    "Zp",
    "Zs",
]

Cc: Final[str]
Cf: Final[str]
Cn: Final[str]
Co: Final[str]
Cs: Final[str]
Ll: Final[str]
Lm: Final[str]
Lo: Final[str]
Lt: Final[str]
Lu: Final[str]
Mc: Final[str]
Me: Final[str]
Mn: Final[str]
Nd: Final[str]
Nl: Final[str]
No: Final[str]
Pc: Final[str]
Pd: Final[str]
Pe: Final[str]
Pf: Final[str]
Pi: Final[str]
Po: Final[str]
Ps: Final[str]
Sc: Final[str]
Sk: Final[str]
Sm: Final[str]
So: Final[str]
Zl: Final[str]
Zp: Final[str]
Zs: Final[str]
xid_continue: Final[str]
xid_start: Final[str]
cats: Final[list[_Cats]]

def combine(*args: _Cats) -> str: ...
def allexcept(*args: _Cats) -> str: ...



from collections.abc import Callable
from typing import Any, Generic, TypeVar

from google.protobuf.descriptor import EnumDescriptor, FieldDescriptor

_T = TypeVar("_T")

def TruncateToFourByteFloat(original: float) -> float: ...
def ToShortestFloat(original: float) -> float: ...
def GetTypeChecker(
    field: FieldDescriptor,
) -> TypeChecker[Any] | IntValueChecker | EnumValueChecker | UnicodeValueChecker | DoubleValueChecker | BoolValueChecker: ...

class TypeChecker(Generic[_T]):
    def __init__(self, *acceptable_types: _T): ...
    def CheckValue(self, proposed_value: _T) -> _T: ...

class TypeCheckerWithDefault(TypeChecker[_T]):
    def __init__(self, default_value: _T, *acceptable_types: _T): ...
    def DefaultValue(self) -> _T: ...

class BoolValueChecker:
    def CheckValue(self, proposed_value: bool) -> bool: ...
    def DefaultValue(self) -> bool: ...

class IntValueChecker:
    def CheckValue(self, proposed_value: int) -> int: ...
    def DefaultValue(self) -> int: ...

class EnumValueChecker:
    def __init__(self, enum_type: EnumDescriptor) -> None: ...
    def CheckValue(self, proposed_value: int) -> int: ...
    def DefaultValue(self) -> int: ...

class UnicodeValueChecker:
    def CheckValue(self, proposed_value: str) -> str: ...
    def DefaultValue(self) -> str: ...

class Int32ValueChecker(IntValueChecker): ...
class Uint32ValueChecker(IntValueChecker): ...
class Int64ValueChecker(IntValueChecker): ...
class Uint64ValueChecker(IntValueChecker): ...

class DoubleValueChecker:
    def CheckValue(self, proposed_value: float) -> float: ...
    def DefaultValue(self) -> float: ...

class FloatValueChecker(DoubleValueChecker):
    def CheckValue(self, proposed_value: float) -> float: ...

TYPE_TO_BYTE_SIZE_FN: dict[int, Callable[..., int]]
TYPE_TO_ENCODER: dict[int, Callable[..., Any]]
TYPE_TO_SIZER: dict[int, Callable[..., Any]]
TYPE_TO_DECODER: dict[int, Callable[..., Any]]
FIELD_TYPE_TO_WIRE_TYPE: dict[int, int]



from _typeshed import Incomplete

class AttrDef:
    name: Incomplete
    key: Incomplete
    validate: Incomplete
    pre_query: Incomplete
    post_query: Incomplete
    default: Incomplete
    dereference_dn: Incomplete
    description: Incomplete
    mandatory: Incomplete
    single_value: Incomplete
    oid_info: Incomplete
    other_names: Incomplete
    def __init__(
        self,
        name,
        key=None,
        validate=None,
        pre_query=None,
        post_query=None,
        default=...,
        dereference_dn=None,
        description=None,
        mandatory: bool = False,
        single_value=None,
        alias=None,
    ) -> None: ...
    def __eq__(self, other): ...
    def __lt__(self, other): ...
    def __hash__(self) -> int: ...
    def __setattr__(self, key: str, value) -> None: ...



import sys
from _heapq import *
from _typeshed import SupportsRichComparison, SupportsRichComparisonT as _T
from collections.abc import Callable, Generator, Iterable
from typing import Final, TypeVar, overload

__all__ = ["heappush", "heappop", "heapify", "heapreplace", "merge", "nlargest", "nsmallest", "heappushpop"]

if sys.version_info >= (3, 14):
    # Added to __all__ in 3.14.1
    __all__ += ["heapify_max", "heappop_max", "heappush_max", "heappushpop_max", "heapreplace_max"]

_S = TypeVar("_S")

__about__: Final[str]

@overload
def merge(*iterables: Iterable[_S], key: Callable[[_S], SupportsRichComparison], reverse: bool = False) -> Generator[_S]: ...
@overload
def merge(*iterables: Iterable[_T], key: None = None, reverse: bool = False) -> Generator[_T]: ...

@overload
def nlargest(n: int, iterable: Iterable[_S], key: Callable[[_S], SupportsRichComparison]) -> list[_S]: ...
@overload
def nlargest(n: int, iterable: Iterable[_T], key: None = None) -> list[_T]: ...

@overload
def nsmallest(n: int, iterable: Iterable[_S], key: Callable[[_S], SupportsRichComparison]) -> list[_S]: ...
@overload
def nsmallest(n: int, iterable: Iterable[_T], key: None = None) -> list[_T]: ...

def _heapify_max(heap: list[SupportsRichComparison], /) -> None: ...  # undocumented



from _typeshed import Incomplete, Unused
from typing import ClassVar, Literal, TypeAlias

from openpyxl.chart.axis import ChartLines, NumericAxis, _BaseAxis
from openpyxl.chart.label import DataLabelList
from openpyxl.chart.updown_bars import UpDownBars
from openpyxl.descriptors.base import Alias, Typed, _ConvertibleToBool
from openpyxl.descriptors.excel import ExtensionList
from openpyxl.descriptors.nested import NestedBool, NestedSet

from ..xml._functions_overloads import _HasTagAndGet
from ._chart import ChartBase

_LineChartBaseGrouping: TypeAlias = Literal["percentStacked", "standard", "stacked"]

class _LineChartBase(ChartBase):
    grouping: NestedSet[_LineChartBaseGrouping]
    varyColors: NestedBool[Literal[True]]
    ser: Incomplete
    dLbls: Typed[DataLabelList, Literal[True]]
    dataLabels: Alias
    dropLines: Typed[ChartLines, Literal[True]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        grouping: _HasTagAndGet[_LineChartBaseGrouping] | _LineChartBaseGrouping = "standard",
        varyColors: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        ser=(),
        dLbls: DataLabelList | None = None,
        dropLines: ChartLines | None = None,
        **kw,
    ) -> None: ...

class LineChart(_LineChartBase):
    tagname: ClassVar[str]
    # Same as parent
    # grouping = _LineChartBase.grouping
    # varyColors = _LineChartBase.varyColors
    # ser = _LineChartBase.ser
    # dLbls = _LineChartBase.dLbls
    # dropLines = _LineChartBase.dropLines
    hiLowLines: Typed[ChartLines, Literal[True]]
    upDownBars: Typed[UpDownBars, Literal[True]]
    marker: NestedBool[Literal[True]]
    smooth: NestedBool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[_BaseAxis, Literal[False]]
    y_axis: Typed[NumericAxis, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        hiLowLines: ChartLines | None = None,
        upDownBars: UpDownBars | None = None,
        marker: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        smooth: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        extLst: Unused = None,
        **kw,
    ) -> None: ...

class LineChart3D(_LineChartBase):
    tagname: ClassVar[str]
    # Same as parent
    # grouping = _LineChartBase.grouping
    # varyColors = _LineChartBase.varyColors
    # ser = _LineChartBase.ser
    # dLbls = _LineChartBase.dLbls
    # dropLines = _LineChartBase.dropLines
    gapDepth: Incomplete
    hiLowLines: Typed[ChartLines, Literal[True]]
    upDownBars: Typed[UpDownBars, Literal[True]]
    marker: NestedBool[Literal[True]]
    smooth: NestedBool[Literal[True]]
    extLst: Typed[ExtensionList, Literal[True]]
    x_axis: Typed[ExtensionList, Literal[False]]
    y_axis: Typed[ExtensionList, Literal[False]]
    z_axis: Typed[ExtensionList, Literal[False]]
    __elements__: ClassVar[tuple[str, ...]]
    def __init__(
        self,
        gapDepth=None,
        hiLowLines: ChartLines | None = None,
        upDownBars: UpDownBars | None = None,
        marker: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        smooth: _HasTagAndGet[_ConvertibleToBool | None] | _ConvertibleToBool | None = None,
        **kw,
    ) -> None: ...



import sys
import types
from abc import ABCMeta
from importlib.machinery import ModuleSpec
from typing_extensions import deprecated

class Loader(metaclass=ABCMeta):
    def load_module(self, fullname: str) -> types.ModuleType: ...
    if sys.version_info < (3, 12):
        @deprecated(
            "Deprecated since Python 3.4; removed in Python 3.12. "
            "The module spec is now used by the import machinery to generate a module repr."
        )
        def module_repr(self, module: types.ModuleType) -> str: ...

    def create_module(self, spec: ModuleSpec) -> types.ModuleType | None: ...
    # Not defined on the actual class for backwards-compatibility reasons,
    # but expected in new code.
    def exec_module(self, module: types.ModuleType) -> None: ...



from _typeshed import Incomplete
from collections.abc import Iterable, Iterator, Sequence
from re import Pattern, RegexFlag
from typing import ClassVar, Final

from pygments.token import _TokenType
from pygments.util import Future

__all__ = [
    "Lexer",
    "RegexLexer",
    "ExtendedRegexLexer",
    "DelegatingLexer",
    "LexerContext",
    "include",
    "inherit",
    "bygroups",
    "using",
    "this",
    "default",
    "words",
    "line_re",
]

line_re: Final[Pattern[str]]

class LexerMeta(type):
    def __new__(cls, name, bases, d): ...

class Lexer(metaclass=LexerMeta):
    name: ClassVar[str]  # Set to None, but always overridden with a non-None value in subclasses.
    aliases: ClassVar[Sequence[str]]  # not intended to be mutable
    filenames: ClassVar[Sequence[str]]
    alias_filenames: ClassVar[Sequence[str]]
    mimetypes: ClassVar[Sequence[str]]
    priority: ClassVar[float]
    url: ClassVar[str]  # Set to None, but always overridden with a non-None value in subclasses.
    version_added: ClassVar[str]  # Set to None, but always overridden with a non-None value in subclasses.
    options: Incomplete
    stripnl: Incomplete
    stripall: Incomplete
    ensurenl: Incomplete
    tabsize: Incomplete
    encoding: Incomplete
    filters: Incomplete
    def __init__(self, **options) -> None: ...
    def add_filter(self, filter_, **options) -> None: ...
    @staticmethod  # @staticmethod added by special handling in metaclass
    def analyse_text(text: str) -> float: ...
    def get_tokens(self, text: str, unfiltered: bool = False) -> Iterator[tuple[_TokenType, str]]: ...
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...

class DelegatingLexer(Lexer):
    root_lexer: Incomplete
    language_lexer: Incomplete
    needle: Incomplete
    def __init__(self, _root_lexer, _language_lexer, _needle=..., **options) -> None: ...
    def get_tokens_unprocessed(self, text: str) -> Iterator[tuple[int, _TokenType, str]]: ...

class include(str): ...
class _inherit: ...

inherit: Incomplete

class combined(tuple[Incomplete, ...]):
    def __new__(cls, *args): ...
    def __init__(self, *args) -> None: ...

class _PseudoMatch:
    def __init__(self, start, text) -> None: ...
    def start(self, arg=None): ...
    def end(self, arg=None): ...
    def group(self, arg=None): ...
    def groups(self): ...
    def groupdict(self): ...

def bygroups(*args): ...

class _This: ...

this: Incomplete

def using(_other, **kwargs): ...

class default:
    state: Incomplete
    def __init__(self, state) -> None: ...

class words(Future):
    words: Incomplete
    prefix: Incomplete
    suffix: Incomplete
    def __init__(self, words, prefix: str = "", suffix: str = "") -> None: ...
    def get(self): ...

class RegexLexerMeta(LexerMeta):
    def process_tokendef(cls, name, tokendefs=None): ...
    def get_tokendefs(cls): ...
    def __call__(cls, *args, **kwds): ...

class RegexLexer(Lexer, metaclass=RegexLexerMeta):
    flags: ClassVar[RegexFlag]
    tokens: ClassVar[dict[str, list[Incomplete]]]
    def get_tokens_unprocessed(self, text: str, stack: Iterable[str] = ("root",)) -> Iterator[tuple[int, _TokenType, str]]: ...

class LexerContext:
    text: Incomplete
    pos: Incomplete
    end: Incomplete
    stack: Incomplete
    def __init__(self, text, pos, stack=None, end=None) -> None: ...

class ExtendedRegexLexer(RegexLexer):
    def get_tokens_unprocessed(  # type: ignore[override]
        self, text: str | None = None, context: LexerContext | None = None
    ) -> Iterator[tuple[int, _TokenType, str]]: ...

class ProfilingRegexLexerMeta(RegexLexerMeta): ...

class ProfilingRegexLexer(RegexLexer, metaclass=ProfilingRegexLexerMeta):
    def get_tokens_unprocessed(self, text: str, stack: Iterable[str] = ("root",)) -> Iterator[tuple[int, _TokenType, str]]: ...



from collections import OrderedDict

from google.protobuf.descriptor_pool import DescriptorPool
from google.protobuf.message import Message

def MakeSimpleProtoClass(
    fields: dict[str, int] | OrderedDict[str, int], full_name: str | None = None, pool: DescriptorPool | None = None
) -> type[Message]: ...



from ..base_client import BaseOAuth, OAuthError as OAuthError
from .apps import DjangoOAuth1App as DjangoOAuth1App, DjangoOAuth2App as DjangoOAuth2App
from .integration import DjangoIntegration as DjangoIntegration, token_update as token_update

class OAuth(BaseOAuth):
    oauth1_client_cls = DjangoOAuth1App
    oauth2_client_cls = DjangoOAuth2App
    framework_integration_cls = DjangoIntegration

__all__ = ["OAuth", "DjangoOAuth1App", "DjangoOAuth2App", "DjangoIntegration", "token_update", "OAuthError"]



import abc
from _typeshed import Incomplete
from typing_extensions import Self

ConnectionError: Incomplete

class GlobalCache(metaclass=abc.ABCMeta):
    __metaclass__: Incomplete
    transient_errors: Incomplete
    strict_read: bool
    strict_write: bool
    @abc.abstractmethod
    def get(self, keys): ...
    @abc.abstractmethod
    def set(self, items, expires: Incomplete | None = ...): ...
    @abc.abstractmethod
    def delete(self, keys): ...
    @abc.abstractmethod
    def watch(self, items): ...
    @abc.abstractmethod
    def unwatch(self, keys): ...
    @abc.abstractmethod
    def compare_and_swap(self, items, expires: Incomplete | None = ...): ...
    @abc.abstractmethod
    def clear(self): ...

class _InProcessGlobalCache(GlobalCache):
    cache: Incomplete
    def __init__(self) -> None: ...
    def get(self, keys): ...
    def set(self, items, expires: Incomplete | None = ...) -> None: ...
    def delete(self, keys) -> None: ...
    def watch(self, items) -> None: ...
    def unwatch(self, keys) -> None: ...
    def compare_and_swap(self, items, expires: Incomplete | None = ...): ...
    def clear(self) -> None: ...

class RedisCache(GlobalCache):
    transient_errors: Incomplete
    @classmethod
    def from_environment(cls, strict_read: bool = ..., strict_write: bool = ...) -> Self: ...
    redis: Incomplete
    strict_read: Incomplete
    strict_write: Incomplete
    def __init__(self, redis, strict_read: bool = ..., strict_write: bool = ...) -> None: ...
    @property
    def pipes(self): ...
    def get(self, keys): ...
    def set(self, items, expires: Incomplete | None = ...) -> None: ...
    def delete(self, keys) -> None: ...
    def watch(self, items) -> None: ...
    def unwatch(self, keys) -> None: ...
    def compare_and_swap(self, items, expires: Incomplete | None = ...): ...
    def clear(self) -> None: ...

class MemcacheCache(GlobalCache):
    class KeyNotSet(Exception):
        key: Incomplete
        def __init__(self, key) -> None: ...
        def __eq__(self, other): ...

    transient_errors: Incomplete
    @classmethod
    def from_environment(cls, max_pool_size: int = ..., strict_read: bool = ..., strict_write: bool = ...) -> Self: ...
    client: Incomplete
    strict_read: Incomplete
    strict_write: Incomplete
    def __init__(self, client, strict_read: bool = ..., strict_write: bool = ...) -> None: ...
    @property
    def caskeys(self): ...
    def get(self, keys): ...
    def set(self, items, expires: Incomplete | None = ...): ...
    def delete(self, keys) -> None: ...
    def watch(self, items) -> None: ...
    def unwatch(self, keys) -> None: ...
    def compare_and_swap(self, items, expires: Incomplete | None = ...): ...
    def clear(self) -> None: ...



from collections.abc import Callable, Container, Iterable, Iterator
from re import Pattern
from typing import Final, Protocol, TypeAlias, type_check_only

from html5lib.filters.base import Filter
from html5lib.filters.sanitizer import Filter as SanitizerFilter
from html5lib.treewalkers.base import TreeWalker

from . import _HTMLAttrKey
from .css_sanitizer import CSSSanitizer
from .html5lib_shim import BleachHTMLParser, BleachHTMLSerializer
from .linkifier import _Token

ALLOWED_TAGS: Final[frozenset[str]]
ALLOWED_ATTRIBUTES: Final[dict[str, list[str]]]
ALLOWED_PROTOCOLS: Final[frozenset[str]]

INVISIBLE_CHARACTERS: Final[str]
INVISIBLE_CHARACTERS_RE: Final[Pattern[str]]
INVISIBLE_REPLACEMENT_CHAR: Final = "?"

class NoCssSanitizerWarning(UserWarning): ...

@type_check_only
class _FilterConstructor(Protocol):
    def __call__(self, *, source: BleachSanitizerFilter) -> Filter: ...

# _FilterConstructor used to be called _Filter
# this alias is obsolete and can potentially be removed in the future
_Filter: TypeAlias = _FilterConstructor  # noqa: Y047

_AttributeFilter: TypeAlias = Callable[[str, str, str], bool]
_AttributeDict: TypeAlias = dict[str, list[str] | _AttributeFilter] | dict[str, list[str]] | dict[str, _AttributeFilter]
_Attributes: TypeAlias = _AttributeFilter | _AttributeDict | list[str]

class Cleaner:
    tags: Iterable[str]
    attributes: _Attributes
    protocols: Iterable[str]
    strip: bool
    strip_comments: bool
    filters: Iterable[_FilterConstructor]
    css_sanitizer: CSSSanitizer | None
    parser: BleachHTMLParser
    walker: TreeWalker
    serializer: BleachHTMLSerializer
    def __init__(
        self,
        tags: Iterable[str] = ...,
        attributes: _Attributes = ...,
        protocols: Iterable[str] = ...,
        strip: bool = False,
        strip_comments: bool = True,
        filters: Iterable[_FilterConstructor] | None = None,
        css_sanitizer: CSSSanitizer | None = None,
    ) -> None: ...
    def clean(self, text: str) -> str: ...

def attribute_filter_factory(attributes: _Attributes) -> _AttributeFilter: ...

class BleachSanitizerFilter(SanitizerFilter):
    allowed_tags: frozenset[str]
    allowed_protocols: frozenset[str]
    attr_filter: _AttributeFilter
    strip_disallowed_tags: bool
    strip_html_comments: bool
    attr_val_is_uri: frozenset[_HTMLAttrKey]
    svg_attr_val_allows_ref: frozenset[_HTMLAttrKey]
    svg_allow_local_href: frozenset[_HTMLAttrKey]
    css_sanitizer: CSSSanitizer | None
    def __init__(
        self,
        source: TreeWalker,
        allowed_tags: Iterable[str] = ...,
        attributes: _Attributes = ...,
        allowed_protocols: Iterable[str] = ...,
        attr_val_is_uri: frozenset[_HTMLAttrKey] = ...,
        svg_attr_val_allows_ref: frozenset[_HTMLAttrKey] = ...,
        svg_allow_local_href: frozenset[_HTMLAttrKey] = ...,
        strip_disallowed_tags: bool = False,
        strip_html_comments: bool = True,
        css_sanitizer: CSSSanitizer | None = None,
    ) -> None: ...
    def sanitize_stream(self, token_iterator: Iterable[_Token]) -> Iterator[_Token]: ...
    def merge_characters(self, token_iterator: Iterable[_Token]) -> Iterator[_Token]: ...
    def __iter__(self) -> Iterator[_Token]: ...
    def sanitize_token(self, token: _Token) -> _Token | list[_Token] | None: ...  # type: ignore[override]
    def sanitize_characters(self, token: _Token) -> _Token | list[_Token]: ...
    def sanitize_uri_value(self, value: str, allowed_protocols: Container[str]) -> str | None: ...
    def allow_token(self, token: _Token) -> _Token: ...
    def disallowed_token(self, token: _Token) -> _Token: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import NoReturn, TypeAlias

from authlib.oauth2.auth import ClientAuth, TokenAuth
from authlib.oauth2.client import OAuth2Client as _OAuth2Client

from ..base_client import OAuthError

__all__ = ["OAuth2Auth", "OAuth2ClientAuth", "AsyncOAuth2Client", "OAuth2Client"]

_Response: TypeAlias = Incomplete  # actual type is httpx.Response
_Request: TypeAlias = Incomplete  # actual type is httpx.Request

# Inherits from httpx.Auth
class OAuth2Auth(TokenAuth):
    requires_request_body: bool
    def auth_flow(self, request: _Request) -> Generator[_Request, _Response]: ...

# Inherits from httpx.Auth
class OAuth2ClientAuth(ClientAuth):
    requires_request_body: bool
    def auth_flow(self, request: _Request) -> Generator[_Request, _Response]: ...

# Inherits from httpx.AsyncClient
class AsyncOAuth2Client(_OAuth2Client):
    SESSION_REQUEST_PARAMS: list[str]
    client_auth_class = OAuth2ClientAuth
    token_auth_class = OAuth2Auth
    oauth_error_class = OAuthError  # type: ignore[assignment]
    def __init__(
        self,
        client_id=None,
        client_secret=None,
        token_endpoint_auth_method=None,
        revocation_endpoint_auth_method=None,
        scope=None,
        redirect_uri=None,
        token=None,
        token_placement="header",
        update_token=None,
        leeway=60,
        **kwargs,
    ) -> None: ...
    async def request(self, method, url, withhold_token: bool = False, auth=..., **kwargs): ...
    async def stream(self, method, url, withhold_token: bool = False, auth=..., **kwargs) -> Generator[Incomplete]: ...
    async def ensure_active_token(self, token): ...  # type: ignore[override]

# Inherits from httpx.Client
class OAuth2Client(_OAuth2Client):
    SESSION_REQUEST_PARAMS: list[str]
    client_auth_class = OAuth2ClientAuth
    token_auth_class = OAuth2Auth
    oauth_error_class = OAuthError  # type: ignore[assignment]
    def __init__(
        self,
        client_id=None,
        client_secret=None,
        token_endpoint_auth_method=None,
        revocation_endpoint_auth_method=None,
        scope=None,
        redirect_uri=None,
        token=None,
        token_placement="header",
        update_token=None,
        **kwargs,
    ) -> None: ...
    @staticmethod
    def handle_error(error_type: str | None, error_description: str | None) -> NoReturn: ...
    def request(self, method, url, withhold_token: bool = False, auth=..., **kwargs): ...
    def stream(self, method, url, withhold_token: bool = False, auth=..., **kwargs): ...



from _typeshed import SupportsRichComparison
from collections.abc import Callable
from typing import Any, Generic, Literal, TypeVar, overload
from typing_extensions import Self

__tracebackhide__: bool

_V = TypeVar("_V", default=Any)

class CollectionMixin(Generic[_V]):
    def is_iterable(self) -> Self: ...
    def is_not_iterable(self) -> Self: ...
    def is_subset_of(self, *supersets: _V) -> Self: ...

    @overload
    def is_sorted(self, key: Callable[[_V], SupportsRichComparison] = ..., reverse: Literal[False] = False) -> Self: ...
    @overload
    def is_sorted(self, *, reverse: Literal[True]) -> Self: ...
    @overload
    def is_sorted(self, key: Callable[[_V], SupportsRichComparison], reverse: Literal[True]) -> Self: ...



from _typeshed import Incomplete
from typing import Literal

from authlib.common.errors import AuthlibHTTPError

def invalid_error_characters(text: str) -> list[str]: ...

class OAuth2Error(AuthlibHTTPError):
    state: Incomplete
    redirect_uri: Incomplete
    redirect_fragment: Incomplete
    def __init__(
        self,
        description: str | None = None,
        uri=None,
        status_code=None,
        state=None,
        redirect_uri=None,
        redirect_fragment: bool = False,
        error=None,
    ) -> None: ...
    def get_body(self) -> list[tuple[Literal["error", "error_description", "error_uri"], str | None]]: ...
    def __call__(self, uri: str | None = None): ...



from _typeshed import Incomplete

class DirSync:
    connection: Incomplete
    base: Incomplete
    filter: Incomplete
    attributes: Incomplete
    cookie: Incomplete
    object_security: Incomplete
    ancestors_first: Incomplete
    public_data_only: Incomplete
    incremental_values: Incomplete
    max_length: Incomplete
    hex_guid: Incomplete
    more_results: bool
    def __init__(
        self,
        connection,
        sync_base,
        sync_filter,
        attributes,
        cookie,
        object_security,
        ancestors_first,
        public_data_only,
        incremental_values,
        max_length,
        hex_guid,
    ) -> None: ...
    def loop(self): ...



from typing import Final, Literal, overload
from typing_extensions import deprecated

from setuptools._distutils.ccompiler import CCompiler

PREFIX: Final[str]
EXEC_PREFIX: Final[str]

@overload
@deprecated("SO is deprecated, use EXT_SUFFIX. Support will be removed when this module is synchronized with stdlib Python 3.11")
def get_config_var(name: Literal["SO"]) -> int | str | None: ...
@overload
def get_config_var(name: str) -> int | str | None: ...

@overload
def get_config_vars() -> dict[str, str | int]: ...
@overload
def get_config_vars(arg: str, /, *args: str) -> list[str | int]: ...

def get_config_h_filename() -> str: ...
def get_makefile_filename() -> str: ...
def get_python_inc(plat_specific: bool = False, prefix: str | None = None) -> str: ...
def get_python_lib(plat_specific: bool = False, standard_lib: bool = False, prefix: str | None = None) -> str: ...
def customize_compiler(compiler: CCompiler) -> None: ...



__all__ = [
    "HasACycle",
    "NodeNotFound",
    "PowerIterationFailedConvergence",
    "ExceededMaxIterations",
    "AmbiguousSolution",
    "NetworkXAlgorithmError",
    "NetworkXException",
    "NetworkXError",
    "NetworkXNoCycle",
    "NetworkXNoPath",
    "NetworkXNotImplemented",
    "NetworkXPointlessConcept",
    "NetworkXUnbounded",
    "NetworkXUnfeasible",
]

class NetworkXException(Exception): ...
class NetworkXError(NetworkXException): ...
class NetworkXPointlessConcept(NetworkXException): ...
class NetworkXAlgorithmError(NetworkXException): ...
class NetworkXUnfeasible(NetworkXAlgorithmError): ...
class NetworkXNoPath(NetworkXUnfeasible): ...
class NetworkXNoCycle(NetworkXUnfeasible): ...
class HasACycle(NetworkXException): ...
class NetworkXUnbounded(NetworkXAlgorithmError): ...
class NetworkXNotImplemented(NetworkXException): ...
class NodeNotFound(NetworkXException): ...
class AmbiguousSolution(NetworkXException): ...
class ExceededMaxIterations(NetworkXException): ...

class PowerIterationFailedConvergence(ExceededMaxIterations):
    def __init__(self, num_iterations, *args, **kw) -> None: ...



from .authorization_server import AuthorizationServer as AuthorizationServer
from .cache import (
    create_exists_nonce_func as create_exists_nonce_func,
    register_nonce_hooks as register_nonce_hooks,
    register_temporary_credential_hooks as register_temporary_credential_hooks,
)
from .resource_protector import ResourceProtector as ResourceProtector, current_credential as current_credential



from _typeshed import Incomplete
from collections.abc import Iterable
from typing import Any

class DirtyApp:
    workers: Incomplete | None

    def init(self) -> None: ...
    def __call__(
        self, action: str, *args: Any, **kwargs: Any
    ) -> Any: ...  # Arguments and result depend on method name passed to action
    def close(self) -> None: ...

def parse_dirty_app_spec(spec: str) -> tuple[str, int | None]: ...
def load_dirty_app(import_path: str): ...
def load_dirty_apps(import_paths: Iterable[str]) -> dict[str, Incomplete]: ...
def get_app_workers_attribute(import_path: str) -> int | None: ...
